/* @copyright Itential, LLC 2019 (pre-modifications) */

// Set globals
/* global describe it log pronghornProps */
/* eslint no-unused-vars: warn */

// include required items for testing & logging
const assert = require('assert');
const fs = require('fs');
const mocha = require('mocha');
const path = require('path');
const winston = require('winston');
const { expect } = require('chai');
const { use } = require('chai');
const td = require('testdouble');

const anything = td.matchers.anything();

// stub and attemptTimeout are used throughout the code so set them here
let logLevel = 'none';
const stub = true;
const isRapidFail = false;
const isSaveMockData = false;
const attemptTimeout = 5000;

// these variables can be changed to run in integrated mode so easier to set them here
// always check these in with bogus data!!!
const host = 'replace.hostorip.here';
const username = 'username';
const password = 'password';
const protocol = 'http';
const port = 80;
const sslenable = false;
const sslinvalid = false;

// these are the adapter properties. You generally should not need to alter
// any of these after they are initially set up
global.pronghornProps = {
  pathProps: {
    encrypted: false
  },
  adapterProps: {
    adapters: [{
      id: 'Test-vmware_nsx_t',
      type: 'VmwareNsxT',
      properties: {
        host,
        port,
        base_path: '',
        version: '',
        cache_location: '',
        protocol,
        stub,
        authentication: {
          auth_method: 'no_authentication',
          username,
          password,
          token: '',
          token_timeout: -1,
          token_cache: 'local',
          invalid_token_error: 401,
          auth_field: 'header.headers.Authorization',
          auth_field_format: 'Basic {b64}{username}:{password}{/b64}'
        },
        healthcheck: {
          type: 'startup',
          frequency: 60000
        },
        throttle: {
          number_pronghorns: 1,
          sync_async: 'sync',
          max_in_queue: 1000,
          concurrent_max: 1,
          expire_timeout: 0,
          avg_runtime: 200
        },
        request: {
          number_retries: 3,
          limit_retry_error: 0,
          failover_codes: [],
          attempt_timeout: attemptTimeout,
          healthcheck_on_timeout: false,
          archiving: false
        },
        proxy: {
          enabled: false,
          host: '',
          port: 1,
          protocol: 'http'
        },
        ssl: {
          ecdhCurve: '',
          enabled: sslenable,
          accept_invalid_cert: sslinvalid,
          ca_file: '',
          secure_protocol: '',
          ciphers: ''
        }
      }
    }]
  }
};

global.$HOME = `${__dirname}/../..`;

// set the log levels that Pronghorn uses, spam and trace are not defaulted in so without
// this you may error on log.trace calls.
const myCustomLevels = {
  levels: {
    spam: 6,
    trace: 5,
    debug: 4,
    info: 3,
    warn: 2,
    error: 1,
    none: 0
  }
};

// need to see if there is a log level passed in
process.argv.forEach((val) => {
  // is there a log level defined to be passed in?
  if (val.indexOf('--LOG') === 0) {
    // get the desired log level
    const inputVal = val.split('=')[1];

    // validate the log level is supported, if so set it
    if (Object.hasOwnProperty.call(myCustomLevels.levels, inputVal)) {
      logLevel = inputVal;
    }
  }
});

// need to set global logging
global.log = new (winston.Logger)({
  level: logLevel,
  levels: myCustomLevels.levels,
  transports: [
    new winston.transports.Console()
  ]
});

/**
 * Runs the common asserts for test
 */
function runCommonAsserts(data, error) {
  assert.equal(undefined, error);
  assert.notEqual(undefined, data);
  assert.notEqual(null, data);
  assert.notEqual(undefined, data.response);
  assert.notEqual(null, data.response);
}

/**
 * Runs the error asserts for the test
 */
function runErrorAsserts(data, error, code, origin, displayStr) {
  assert.equal(null, data);
  assert.notEqual(undefined, error);
  assert.notEqual(null, error);
  assert.notEqual(undefined, error.IAPerror);
  assert.notEqual(null, error.IAPerror);
  assert.notEqual(undefined, error.IAPerror.displayString);
  assert.notEqual(null, error.IAPerror.displayString);
  assert.equal(code, error.icode);
  assert.equal(origin, error.IAPerror.origin);
  assert.equal(displayStr, error.IAPerror.displayString);
}

/**
 * @function saveMockData
 * Attempts to take data from responses and place them in MockDataFiles to help create Mockdata.
 * Note, this was built based on entity file structure for Adapter-Engine 1.6.x
 * @param {string} entityName - Name of the entity saving mock data for
 * @param {string} actionName -  Name of the action saving mock data for
 * @param {string} descriptor -  Something to describe this test (used as a type)
 * @param {string or object} responseData - The data to put in the mock file.
 */
function saveMockData(entityName, actionName, descriptor, responseData) {
  // do not need to save mockdata if we are running in stub mode (already has mock data) or if told not to save
  if (stub || !isSaveMockData) {
    return false;
  }

  // must have a response in order to store the response
  if (responseData && responseData.response) {
    const data = responseData.response;

    try {
      const base = `./entities/${entityName}/`;
      const filename = `mockdatafiles/${actionName}-${descriptor}.json`;

      // write the data we retrieved
      fs.writeFile(base + filename, JSON.stringify(data, null, 2), 'utf8', (errWritingMock) => {
        if (errWritingMock) throw errWritingMock;

        // update the action file to reflect the changes. Note: We're replacing the default object for now!
        fs.readFile(`${base}action.json`, (errRead, content) => {
          if (errRead) throw errRead;

          // parse the action file into JSON
          const parsedJson = JSON.parse(content);

          // The object update we'll write in.
          const responseObj = {
            type: descriptor,
            key: '',
            mockFile: filename
          };

          // get the object for method we're trying to change.
          const currentMethodAction = parsedJson.actions.find(obj => obj.name === actionName);

          // if the method was not found - should never happen but...
          if (!currentMethodAction) {
            throw Error('Can\'t find an action for this method in the provided entity.');
          }

          // if there is a response object, we want to replace the Response object. Otherwise we'll create one.
          const actionResponseObj = currentMethodAction.responseObjects.find(obj => obj.type === descriptor);

          // Add the action responseObj back into the array of response objects.
          if (!actionResponseObj) {
            // if there is a default response object, we want to get the key.
            const defaultResponseObj = currentMethodAction.responseObjects.find(obj => obj.type === 'default');

            // save the default key into the new response object
            if (!defaultResponseObj) {
              responseObj.key = defaultResponseObj.key;
            }

            // save the new response object
            currentMethodAction.responseObjects = [responseObj];
          } else {
            // update the location of the mock data file
            actionResponseObj.mockFile = responseObj.mockFile;
          }

          // Save results
          fs.writeFile(`${base}action.json`, JSON.stringify(parsedJson, null, 2), (err) => {
            if (err) throw err;
          });
        });
      });
    } catch (e) {
      log.debug(`Failed to save mock data for ${actionName}. ${e.message}`);
      return false;
    }
  }

  // no response to save
  log.debug(`No data passed to save into mockdata for ${actionName}`);
  return false;
}


// require the adapter that we are going to be using
const VmwareNsxT = require('../../adapter.js');

// begin the testing - these should be pretty well defined between the describe and the it!
describe('[integration] Vmware_nsx_t Adapter Test', () => {
  describe('VmwareNsxT Class Tests', () => {
    const a = new VmwareNsxT(
      pronghornProps.adapterProps.adapters[0].id,
      pronghornProps.adapterProps.adapters[0].properties
    );

    if (isRapidFail) {
      const state = {};
      state.passed = true;

      mocha.afterEach(function x() {
        state.passed = state.passed
        && (this.currentTest.state === 'passed');
      });
      mocha.beforeEach(function x() {
        if (!state.passed) {
          return this.currentTest.skip();
        }
        return true;
      });
    }

    describe('#class instance created', () => {
      it('should be a class with properties', (done) => {
        assert.notEqual(null, a);
        assert.notEqual(undefined, a);
        assert.notEqual(null, a.allProps);
        const check = global.pronghornProps.adapterProps.adapters[0].properties.healthcheck.type;
        assert.equal(check, a.healthcheckType);
        done();
      }).timeout(attemptTimeout);
    });

    describe('#connect', () => {
      it('should get connected - no healthcheck', (done) => {
        a.healthcheckType = 'none';
        a.connect();
        assert.equal(true, a.alive);
        done();
      });
      it('should get connected - startup healthcheck', (done) => {
        a.healthcheckType = 'startup';
        a.connect();
        assert.equal(true, a.alive);
        done();
      });
    });

    describe('#healthCheck', () => {
      it('should be healthy', (done) => {
        const p = new Promise((resolve) => {
          a.healthCheck(null, (data) => {
            assert.equal(true, a.healthy);
            saveMockData('system', 'healthcheck', 'default', data);
            resolve(data);
            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    /*
    -----------------------------------------------------------------------
    -----------------------------------------------------------------------
    *** All code above this comment will be replaced during a migration ***
    ******************* DO NOT REMOVE THIS COMMENT BLOCK ******************
    -----------------------------------------------------------------------
    -----------------------------------------------------------------------
    */
    /* eslint no-underscore-dangle:warn */

    describe('#putHpmfeaturesfeatureStackName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putHpmfeaturesfeatureStackName('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.feature_stack_name);
              assert.equal(true, Array.isArray(data.response.client_type_collection_configurations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getHpmfeaturesfeatureStackName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getHpmfeaturesfeatureStackName('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.feature_stack_name);
              assert.equal(true, Array.isArray(data.response.client_type_collection_configurations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postHpmfeaturesfeatureStackNameactionresetCollectionFrequency - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postHpmfeaturesfeatureStackNameactionresetCollectionFrequency('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.feature_stack_name);
              assert.equal(true, Array.isArray(data.response.client_type_collection_configurations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putHpmglobalConfig - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putHpmglobalConfig('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.modified_feature_stack_collection_configurations);
              assert.equal(true, data.response.is_data_collection_enabled);
              assert.equal(3, data.response.aggregated_data_collection_frequency);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getHpmglobalConfig - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getHpmglobalConfig((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.modified_feature_stack_collection_configurations);
              assert.equal(false, data.response.is_data_collection_enabled);
              assert.equal(5, data.response.aggregated_data_collection_frequency);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getHpmfeatures - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getHpmfeatures((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalSwitcheslswitchIdvtepTable - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalSwitcheslswitchIdvtepTable('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(9, data.response.last_update_timestamp);
              assert.equal('string', data.response.logical_switch_id);
              assert.equal(true, Array.isArray(data.response.results));
              assert.equal('string', data.response.transport_node_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getSwitchingProfilesswitchingProfileIdsummary - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getSwitchingProfilesswitchingProfileIdsummary('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(3, data.response.num_logical_ports);
              assert.equal('string', data.response.switching_profile_id);
              assert.equal(5, data.response.num_logical_switches);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLogicalSwitches - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLogicalSwitches('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('MTEP', data.response.replication_mode);
              assert.equal('UP', data.response.admin_state);
              assert.equal(true, Array.isArray(data.response.address_bindings));
              assert.equal('string', data.response.transport_zone_id);
              assert.equal('string', data.response.ip_pool_id);
              assert.equal(3, data.response.vlan);
              assert.equal(true, Array.isArray(data.response.switching_profile_ids));
              assert.equal('string', data.response.mac_pool_id);
              assert.equal(7, data.response.vni);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalSwitches - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalSwitches('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLogicalPorts - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLogicalPorts('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.logical_switch_id);
              assert.equal(true, Array.isArray(data.response.switching_profile_ids));
              assert.equal('object', typeof data.response.attachment);
              assert.equal('UP', data.response.admin_state);
              assert.equal(true, Array.isArray(data.response.address_bindings));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalPorts - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalPorts('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalPortslportIdstate - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalPortslportIdstate('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.transport_node_ids));
              assert.equal(true, Array.isArray(data.response.discovered_bindings));
              assert.equal('string', data.response.id);
              assert.equal(true, Array.isArray(data.response.realized_bindings));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalSwitcheslswitchIdsummary - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalSwitcheslswitchIdsummary('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(1, data.response.num_logical_ports);
              assert.equal('string', data.response.logical_switch_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalSwitcheslswitchIdmacTableformatcsv - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalSwitcheslswitchIdmacTableformatcsv('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.file_name);
              assert.equal(9, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalSwitcheslswitchIdstate - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalSwitcheslswitchIdstate('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('orphaned', data.response.state);
              assert.equal(true, Array.isArray(data.response.details));
              assert.equal(6, data.response.failure_code);
              assert.equal('string', data.response.failure_message);
              assert.equal('string', data.response.logical_switch_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalSwitcheslswitchIdstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalSwitcheslswitchIdstatistics('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.tx_bytes);
              assert.equal('object', typeof data.response.rx_packets);
              assert.equal('object', typeof data.response.tx_packets);
              assert.equal('object', typeof data.response.rx_bytes);
              assert.equal('object', typeof data.response.mac_learning);
              assert.equal('object', typeof data.response.dropped_by_security_packets);
              assert.equal(9, data.response.last_update_timestamp);
              assert.equal('string', data.response.logical_switch_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalSwitchesstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalSwitchesstatus('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(10, data.response.last_update_timestamp);
              assert.equal(1, data.response.total_switches);
              assert.equal(true, Array.isArray(data.response.filters));
              assert.equal(5, data.response.fully_realized_switches);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putSwitchingProfilesswitchingProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putSwitchingProfilesswitchingProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.required_capabilities));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getSwitchingProfilesswitchingProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getSwitchingProfilesswitchingProfileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.required_capabilities));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteSwitchingProfilesswitchingProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteSwitchingProfilesswitchingProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postSwitchingProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postSwitchingProfiles('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.required_capabilities));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getSwitchingProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getSwitchingProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalPortslportIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalPortslportIdstatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.logical_port_id);
              assert.equal('UNKNOWN', data.response.status);
              assert.equal(7, data.response.last_update_timestamp);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalPortslportIdmacTableformatcsv - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalPortslportIdmacTableformatcsv('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.file_name);
              assert.equal(2, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalPortslportIdmacTable - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalPortslportIdmacTable('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal('string', data.response.logical_port_id);
              assert.equal(5, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.results));
              assert.equal('string', data.response.transport_node_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalSwitcheslswitchIdvtepTableformatcsv - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalSwitcheslswitchIdvtepTableformatcsv('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.file_name);
              assert.equal(6, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalSwitcheslswitchIdmacTable - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalSwitcheslswitchIdmacTable('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(6, data.response.last_update_timestamp);
              assert.equal('string', data.response.logical_switch_id);
              assert.equal(true, Array.isArray(data.response.results));
              assert.equal('string', data.response.transport_node_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalPortsstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalPortsstatus('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(1, data.response.total_ports);
              assert.equal(1, data.response.last_update_timestamp);
              assert.equal(9, data.response.up_ports);
              assert.equal(true, Array.isArray(data.response.filters));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalPortslportIdstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalPortslportIdstatistics('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.tx_bytes);
              assert.equal('object', typeof data.response.rx_packets);
              assert.equal('object', typeof data.response.tx_packets);
              assert.equal('object', typeof data.response.rx_bytes);
              assert.equal('object', typeof data.response.mac_learning);
              assert.equal('object', typeof data.response.dropped_by_security_packets);
              assert.equal('string', data.response.logical_port_id);
              assert.equal(7, data.response.last_update_timestamp);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalSwitcheslswitchId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalSwitcheslswitchId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('MTEP', data.response.replication_mode);
              assert.equal('UP', data.response.admin_state);
              assert.equal(true, Array.isArray(data.response.address_bindings));
              assert.equal('string', data.response.transport_zone_id);
              assert.equal('string', data.response.ip_pool_id);
              assert.equal(7, data.response.vlan);
              assert.equal(true, Array.isArray(data.response.switching_profile_ids));
              assert.equal('string', data.response.mac_pool_id);
              assert.equal(9, data.response.vni);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLogicalSwitcheslswitchId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLogicalSwitcheslswitchId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalSwitcheslswitchId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalSwitcheslswitchId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('SOURCE', data.response.replication_mode);
              assert.equal('DOWN', data.response.admin_state);
              assert.equal(true, Array.isArray(data.response.address_bindings));
              assert.equal('string', data.response.transport_zone_id);
              assert.equal('string', data.response.ip_pool_id);
              assert.equal(7, data.response.vlan);
              assert.equal(true, Array.isArray(data.response.switching_profile_ids));
              assert.equal('string', data.response.mac_pool_id);
              assert.equal(10, data.response.vni);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalSwitchesstate - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalSwitchesstate('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalPortslportId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalPortslportId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.logical_switch_id);
              assert.equal(true, Array.isArray(data.response.switching_profile_ids));
              assert.equal('object', typeof data.response.attachment);
              assert.equal('UP', data.response.admin_state);
              assert.equal(true, Array.isArray(data.response.address_bindings));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalPortslportId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalPortslportId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.logical_switch_id);
              assert.equal(true, Array.isArray(data.response.switching_profile_ids));
              assert.equal('object', typeof data.response.attachment);
              assert.equal('DOWN', data.response.admin_state);
              assert.equal(true, Array.isArray(data.response.address_bindings));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLogicalPortslportId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLogicalPortslportId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAppDiscoverysessionssessionIdinstalledApps - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAppDiscoverysessionssessionIdinstalledApps('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postAppDiscoverysessionssessionIdreClassify - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postAppDiscoverysessionssessionIdreClassify('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('RUNNING', data.response.status);
              assert.equal(4, data.response.start_timestamp);
              assert.equal(7, data.response.end_timestamp);
              assert.equal('NOT_REQUIRED', data.response.reclassification);
              assert.equal(true, Array.isArray(data.response.app_profile_summary_list));
              assert.equal(true, Array.isArray(data.response.ns_groups));
              assert.equal(true, Array.isArray(data.response.app_profiles));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postAppDiscoverysessionssessionIdreportappInfoAndVmformatcsv - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postAppDiscoverysessionssessionIdreportappInfoAndVmformatcsv('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.file_name);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAppDiscoverysessionssessionIdnsGroups - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAppDiscoverysessionssessionIdnsGroups('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAppDiscoverysessionssessionIdsummary - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAppDiscoverysessionssessionIdsummary('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('FAILED', data.response.status);
              assert.equal(6, data.response.start_timestamp);
              assert.equal(8, data.response.end_timestamp);
              assert.equal('REQUIRED', data.response.reclassification);
              assert.equal(true, Array.isArray(data.response.app_profile_summary_list));
              assert.equal(true, Array.isArray(data.response.ns_groups));
              assert.equal(true, Array.isArray(data.response.app_profiles));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postAppDiscoveryappProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postAppDiscoveryappProfiles('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.app_profile_category);
              assert.equal(false, data.response.default_app_profile);
              assert.equal(true, Array.isArray(data.response.app_profile_criteria));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAppDiscoveryappProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAppDiscoveryappProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAppDiscoverysessionssessionIdreportappProfileAndAppInfoformatcsv - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAppDiscoverysessionssessionIdreportappProfileAndAppInfoformatcsv('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.file_name);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteAppDiscoverysessionssessionId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteAppDiscoverysessionssessionId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAppDiscoverysessionssessionId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAppDiscoverysessionssessionId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('FAILED', data.response.status);
              assert.equal(9, data.response.start_timestamp);
              assert.equal(10, data.response.end_timestamp);
              assert.equal('NOT_REQUIRED', data.response.reclassification);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAppDiscoverysessionssessionIdappProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAppDiscoverysessionssessionIdappProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAppDiscoverysessionssessionIdnsGroupsnsGroupIdmembers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAppDiscoverysessionssessionIdnsGroupsnsGroupIdmembers('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putAppDiscoveryappProfilesappProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putAppDiscoveryappProfilesappProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.app_profile_category);
              assert.equal(true, data.response.default_app_profile);
              assert.equal(true, Array.isArray(data.response.app_profile_criteria));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAppDiscoveryappProfilesappProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAppDiscoveryappProfilesappProfileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.app_profile_category);
              assert.equal(false, data.response.default_app_profile);
              assert.equal(true, Array.isArray(data.response.app_profile_criteria));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteAppDiscoveryappProfilesappProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteAppDiscoveryappProfilesappProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postAppDiscoverysessions - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postAppDiscoverysessions('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('FINISHED', data.response.status);
              assert.equal(9, data.response.start_timestamp);
              assert.equal(7, data.response.end_timestamp);
              assert.equal('NOT_REQUIRED', data.response.reclassification);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAppDiscoverysessions - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAppDiscoverysessions('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteMacSetsmacSetIdmembersmacAddress - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteMacSetsmacSetIdmembersmacAddress('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNsServicesnsServiceId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNsServicesnsServiceId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.default_service);
              assert.equal('object', typeof data.response.nsservice_element);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteNsServicesnsServiceId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteNsServicesnsServiceId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsServicesnsServiceId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsServicesnsServiceId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.default_service);
              assert.equal('object', typeof data.response.nsservice_element);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postMacSetsmacSetIdmembers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postMacSetsmacSetIdmembers('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal('string', data.response.mac_address);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getMacSetsmacSetIdmembers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getMacSetsmacSetIdmembers('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsGroupsnsgroupIdserviceAssociations - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsGroupsnsgroupIdserviceAssociations('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal('DNEServiceAssociationListResult', data.response.service_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsGroupsnsGroupIdeffectiveIpAddressMembers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsGroupsnsGroupIdeffectiveIpAddressMembers('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postIpSets - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postIpSets('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.ip_addresses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getIpSets - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getIpSets('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsGroupsnsGroupIdeffectiveLogicalSwitchMembers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsGroupsnsGroupIdeffectiveLogicalSwitchMembers('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsGroupsnsGroupIdmemberTypes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsGroupsnsGroupIdmemberTypes('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.results));
              assert.equal(3, data.response.result_count);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNsGroupsnsGroupId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNsGroupsnsGroupId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(2, data.response.member_count);
              assert.equal(true, Array.isArray(data.response.members));
              assert.equal(true, Array.isArray(data.response.membership_criteria));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNsGroupsnsGroupId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNsGroupsnsGroupId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(9, data.response.member_count);
              assert.equal(true, Array.isArray(data.response.members));
              assert.equal(true, Array.isArray(data.response.membership_criteria));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsGroupsnsGroupId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsGroupsnsGroupId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(5, data.response.member_count);
              assert.equal(true, Array.isArray(data.response.members));
              assert.equal(true, Array.isArray(data.response.membership_criteria));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteNsGroupsnsGroupId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteNsGroupsnsGroupId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsGroupsnsGroupIdeffectiveVirtualMachineMembers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsGroupsnsGroupIdeffectiveVirtualMachineMembers('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNsServiceGroupsnsServiceGroupId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNsServiceGroupsnsServiceGroupId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.default_service);
              assert.equal('NON_ETHER', data.response.service_type);
              assert.equal(true, Array.isArray(data.response.members));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteNsServiceGroupsnsServiceGroupId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteNsServiceGroupsnsServiceGroupId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsServiceGroupsnsServiceGroupId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsServiceGroupsnsServiceGroupId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.default_service);
              assert.equal('NON_ETHER', data.response.service_type);
              assert.equal(true, Array.isArray(data.response.members));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putIpSetsipSetId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putIpSetsipSetId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.ip_addresses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteIpSetsipSetId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteIpSetsipSetId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getIpSetsipSetId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getIpSetsipSetId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.ip_addresses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putMacSetsmacSetId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putMacSetsmacSetId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.mac_addresses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteMacSetsmacSetId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteMacSetsmacSetId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getMacSetsmacSetId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getMacSetsmacSetId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.mac_addresses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNsServices - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNsServices('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.default_service);
              assert.equal('object', typeof data.response.nsservice_element);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsServices - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsServices('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsGroupsunassociatedVirtualMachines - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsGroupsunassociatedVirtualMachines('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(3, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postMacSets - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postMacSets('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.mac_addresses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getMacSets - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getMacSets('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsGroupsnsGroupIdeffectiveLogicalPortMembers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsGroupsnsGroupIdeffectiveLogicalPortMembers('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNsServiceGroups - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNsServiceGroups('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.default_service);
              assert.equal('ETHER', data.response.service_type);
              assert.equal(true, Array.isArray(data.response.members));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsServiceGroups - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsServiceGroups('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNsGroups - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNsGroups('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(3, data.response.member_count);
              assert.equal(true, Array.isArray(data.response.members));
              assert.equal(true, Array.isArray(data.response.membership_criteria));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNsGroups - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNsGroups('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putIpfixObsPointsswitchGlobal - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putIpfixObsPointsswitchGlobal('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(300, data.response.idle_timeout);
              assert.equal(5, data.response.observation_domain_id);
              assert.equal(true, Array.isArray(data.response.collectors));
              assert.equal(300, data.response.active_timeout);
              assert.equal(0.1, data.response.packet_sample_probability);
              assert.equal(false, data.response.enabled);
              assert.equal(16384, data.response.max_flows);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getIpfixObsPointsswitchGlobal - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getIpfixObsPointsswitchGlobal((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(300, data.response.idle_timeout);
              assert.equal(1, data.response.observation_domain_id);
              assert.equal(true, Array.isArray(data.response.collectors));
              assert.equal(300, data.response.active_timeout);
              assert.equal(0.1, data.response.packet_sample_probability);
              assert.equal(true, data.response.enabled);
              assert.equal(16384, data.response.max_flows);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteTraceflowstraceflowId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteTraceflowstraceflowId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTraceflowstraceflowId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTraceflowstraceflowId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('IN_PROGRESS', data.response.operation_state);
              assert.equal('object', typeof data.response.logical_counters);
              assert.equal(8, data.response.timeout);
              assert.equal('string', data.response.lport_id);
              assert.equal('object', typeof data.response.counters);
              assert.equal(false, data.response.result_overflowed);
              assert.equal(true, Array.isArray(data.response.analysis));
              assert.equal('string', data.response.id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportZoneszoneIdtransportNodeStatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportZoneszoneIdtransportNodeStatus('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTraceflows - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postTraceflows('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('IN_PROGRESS', data.response.operation_state);
              assert.equal('object', typeof data.response.logical_counters);
              assert.equal(2, data.response.timeout);
              assert.equal('string', data.response.lport_id);
              assert.equal('object', typeof data.response.counters);
              assert.equal(true, data.response.result_overflowed);
              assert.equal(true, Array.isArray(data.response.analysis));
              assert.equal('string', data.response.id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTraceflows - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTraceflows('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodelogslogName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodelogslogName('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response.last_modified_time);
              assert.equal(10, data.response.log_size);
              assert.equal('string', data.response.log_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodelogslogNamedata - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.getNodelogslogNamedata('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportNodesnodeIdpnicBondStatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportNodesnodeIdpnicBondStatus('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodelogs - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodelogs((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportNodesstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportNodesstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(4, data.response.degraded_count);
              assert.equal(1, data.response.down_count);
              assert.equal(6, data.response.unknown_count);
              assert.equal(8, data.response.up_count);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportNodesnodeIdremoteTransportNodeStatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportNodesnodeIdremoteTransportNodeStatus('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportNodesnodeIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportNodesnodeIdstatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('DOWN', data.response.status);
              assert.equal('string', data.response.node_uuid);
              assert.equal('DOWN', data.response.mgmt_connection_status);
              assert.equal('object', typeof data.response.control_connection_status);
              assert.equal('object', typeof data.response.pnic_status);
              assert.equal('string', data.response.node_display_name);
              assert.equal('object', typeof data.response.tunnel_status);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postMirrorSessions - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postMirrorSessions('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('INGRESS', data.response.direction);
              assert.equal(3, data.response.encapsulation_vlan_id);
              assert.equal(5, data.response.snap_length);
              assert.equal(true, Array.isArray(data.response.mirror_sources));
              assert.equal(true, data.response.preserve_original_vlan);
              assert.equal('object', typeof data.response.mirror_destination);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getMirrorSessions - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getMirrorSessions('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putMirrorSessionsmirrorSessionId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putMirrorSessionsmirrorSessionId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('INGRESS', data.response.direction);
              assert.equal(2, data.response.encapsulation_vlan_id);
              assert.equal(7, data.response.snap_length);
              assert.equal(true, Array.isArray(data.response.mirror_sources));
              assert.equal(false, data.response.preserve_original_vlan);
              assert.equal('object', typeof data.response.mirror_destination);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getMirrorSessionsmirrorSessionId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getMirrorSessionsmirrorSessionId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('INGRESS', data.response.direction);
              assert.equal(10, data.response.encapsulation_vlan_id);
              assert.equal(6, data.response.snap_length);
              assert.equal(true, Array.isArray(data.response.mirror_sources));
              assert.equal(true, data.response.preserve_original_vlan);
              assert.equal('object', typeof data.response.mirror_destination);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteMirrorSessionsmirrorSessionId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteMirrorSessionsmirrorSessionId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalPortslportIdforwardingPath - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalPortslportIdforwardingPath('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.containers);
              assert.equal(true, Array.isArray(data.response.hypervisors));
              assert.equal(true, Array.isArray(data.response.errors));
              assert.equal(true, Array.isArray(data.response.logical_switches));
              assert.equal(true, Array.isArray(data.response.edge_node_groups));
              assert.equal(true, Array.isArray(data.response.routers));
              assert.equal(true, Array.isArray(data.response.vms));
              assert.equal(true, Array.isArray(data.response.tunnels));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTraceflowstraceflowIdobservations - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTraceflowstraceflowIdobservations('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportZonesstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportZonesstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(3, data.response.degraded_count);
              assert.equal(9, data.response.down_count);
              assert.equal(7, data.response.unknown_count);
              assert.equal(8, data.response.up_count);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getIpfixObsPoints - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getIpfixObsPoints((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportZoneszoneIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportZoneszoneIdstatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(10, data.response.degraded_count);
              assert.equal(3, data.response.down_count);
              assert.equal(5, data.response.unknown_count);
              assert.equal(8, data.response.up_count);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportZoneszoneIdtransportNodeStatusReport - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportZoneszoneIdtransportNodeStatusReport('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportZonestransportNodeStatusReport - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportZonestransportNodeStatusReport('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postMirrorSessionsmirrorSessionIdactionverify - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postMirrorSessionsmirrorSessionIdactionverify('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportZonestransportNodeStatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportZonestransportNodeStatus('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLogicalRouterslogicalRouterIdroutingstaticRoutes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLogicalRouterslogicalRouterIdroutingstaticRoutes('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.next_hops));
              assert.equal('string', data.response.logical_router_id);
              assert.equal('string', data.response.network);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingstaticRoutes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingstaticRoutes('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLogicalRouters - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLogicalRouters('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('NON_PREEMPTIVE', data.response.failover_mode);
              assert.equal(true, Array.isArray(data.response.firewall_sections));
              assert.equal('object', typeof data.response.advanced_config);
              assert.equal('TIER1', data.response.router_type);
              assert.equal(1, data.response.preferred_edge_cluster_member_index);
              assert.equal('ACTIVE_STANDBY', data.response.high_availability_mode);
              assert.equal('string', data.response.edge_cluster_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouters - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouters('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingforwardingTable - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingforwardingTable('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal('string', data.response.logical_router_name);
              assert.equal(9, data.response.last_update_timestamp);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLogicalRouterslogicalRouterIdroutingrouteMaps - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLogicalRouterslogicalRouterIdroutingrouteMaps('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, Array.isArray(data.response.sequences));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingrouteMaps - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingrouteMaps('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdnatrulesruleIdstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdnatrulesruleIdstatistics('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(5, data.response.total_packets);
              assert.equal(2, data.response.total_bytes);
              assert.equal(1, data.response.active_sessions);
              assert.equal(1, data.response.last_update_timestamp);
              assert.equal('string', data.response.logical_router_id);
              assert.equal('string', data.response.id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingrouteTableformatcsv - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingrouteTableformatcsv('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.file_name);
              assert.equal(4, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdroutingstaticRoutesbfdPeersbfdPeerId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdroutingstaticRoutesbfdPeersbfdPeerId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.bfd_config);
              assert.equal(true, data.response.enabled);
              assert.equal('string', data.response.peer_ip_address);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingstaticRoutesbfdPeersbfdPeerId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingstaticRoutesbfdPeersbfdPeerId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.bfd_config);
              assert.equal(true, data.response.enabled);
              assert.equal('string', data.response.peer_ip_address);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLogicalRouterslogicalRouterIdroutingstaticRoutesbfdPeersbfdPeerId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLogicalRouterslogicalRouterIdroutingstaticRoutesbfdPeersbfdPeerId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdstatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(7, data.response.last_update_timestamp);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, Array.isArray(data.response.per_node_status));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLogicalRouterslogicalRouterIdroutingipPrefixLists - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLogicalRouterslogicalRouterIdroutingipPrefixLists('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.prefixes));
              assert.equal('string', data.response.logical_router_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingipPrefixLists - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingipPrefixLists('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdroutingredistribution - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdroutingredistribution('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(false, data.response.bgp_enabled);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingredistribution - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingredistribution('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(false, data.response.bgp_enabled);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putDhcprelaysrelayId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putDhcprelaysrelayId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.dhcp_relay_profile_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcprelaysrelayId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcprelaysrelayId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.dhcp_relay_profile_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteDhcprelaysrelayId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteDhcprelaysrelayId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdroutingstaticRoutesid - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdroutingstaticRoutesid('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.next_hops));
              assert.equal('string', data.response.logical_router_id);
              assert.equal('string', data.response.network);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingstaticRoutesid - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingstaticRoutesid('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.next_hops));
              assert.equal('string', data.response.logical_router_id);
              assert.equal('string', data.response.network);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLogicalRouterslogicalRouterIdroutingstaticRoutesid - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLogicalRouterslogicalRouterIdroutingstaticRoutesid('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putDhcprelayProfilesrelayProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putDhcprelayProfilesrelayProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.server_addresses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteDhcprelayProfilesrelayProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteDhcprelayProfilesrelayProfileId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcprelayProfilesrelayProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcprelayProfilesrelayProfileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.server_addresses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLogicalRouterslogicalRouterIdactionreprocess - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postLogicalRouterslogicalRouterIdactionreprocess('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIddebugInfoformattext - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIddebugInfoformattext('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingbgpneighborsstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingbgpneighborsstatus('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal('string', data.response.logical_router_name);
              assert.equal(4, data.response.last_update_timestamp);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdroutingredistributionrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdroutingredistributionrules('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.rules));
              assert.equal('string', data.response.logical_router_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingredistributionrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingredistributionrules('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.rules));
              assert.equal('string', data.response.logical_router_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postServices - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postServices('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('DhcpRelayService', data.response.resource_type);
              assert.equal('string', data.response.dhcp_relay_profile_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getServices - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getServices('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postDhcprelayProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postDhcprelayProfiles('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.server_addresses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcprelayProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcprelayProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingrouteTable - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingrouteTable('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal('string', data.response.logical_router_name);
              assert.equal(4, data.response.last_update_timestamp);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLogicalRouterslogicalRouterIdroutingbgpneighbors - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLogicalRouterslogicalRouterIdroutingbgpneighbors('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.filter_in_ipprefixlist_id);
              assert.equal('string', data.response.neighbor_address);
              assert.equal(3, data.response.remote_as);
              assert.equal('string', data.response.filter_out_ipprefixlist_id);
              assert.equal(180, data.response.hold_down_timer);
              assert.equal(1, data.response.maximum_hop_limit);
              assert.equal(true, data.response.enabled);
              assert.equal('object', typeof data.response.bfd_config);
              assert.equal('string', data.response.logical_router_id);
              assert.equal('string', data.response.remote_as_num);
              assert.equal('string', data.response.filter_out_routemap_id);
              assert.equal('string', data.response.filter_in_routemap_id);
              assert.equal(60, data.response.keep_alive_timer);
              assert.equal('string', data.response.password);
              assert.equal('string', data.response.source_address);
              assert.equal(true, Array.isArray(data.response.source_addresses));
              assert.equal(true, data.response.enable_bfd);
              assert.equal(true, Array.isArray(data.response.address_families));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingbgpneighbors - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingbgpneighbors('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingforwardingTableformatcsv - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingforwardingTableformatcsv('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.file_name);
              assert.equal(2, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingroutingTable - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingroutingTable('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal('string', data.response.logical_router_name);
              assert.equal(2, data.response.last_update_timestamp);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterPortslogicalRouterPortIdarpTable - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterPortslogicalRouterPortIdarpTable('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(6, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.results));
              assert.equal('string', data.response.logical_router_port_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterPortslogicalRouterPortIdstatisticssummary - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterPortslogicalRouterPortIdstatisticssummary('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(6, data.response.last_update_timestamp);
              assert.equal('object', typeof data.response.rx);
              assert.equal('object', typeof data.response.tx);
              assert.equal('string', data.response.logical_router_port_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postServiceProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postServiceProfiles('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('DhcpRelayProfile', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.server_addresses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getServiceProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getServiceProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLogicalRouterPorts - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLogicalRouterPorts('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LogicalRouterLinkPortOnTIER1', data.response.resource_type);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, Array.isArray(data.response.service_bindings));
              assert.equal('string', data.response.mac_address);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterPorts - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterPorts('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdnatrulesstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdnatrulesstatistics('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(6, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.per_transport_node_statistics));
              assert.equal('object', typeof data.response.statistics_across_all_nodes);
              assert.equal('string', data.response.logical_router_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportNodesnodeIdstatisticsnatRules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportNodesnodeIdstatisticsnatRules('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(9, data.response.total_packets);
              assert.equal(3, data.response.total_bytes);
              assert.equal(4, data.response.active_sessions);
              assert.equal(9, data.response.last_update_timestamp);
              assert.equal('string', data.response.transport_node_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLogicalRouterslogicalRouterIdroutingstaticRoutesbfdPeers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLogicalRouterslogicalRouterIdroutingstaticRoutesbfdPeers('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.bfd_config);
              assert.equal(true, data.response.enabled);
              assert.equal('string', data.response.peer_ip_address);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingstaticRoutesbfdPeers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingstaticRoutesbfdPeers('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('NON_PREEMPTIVE', data.response.failover_mode);
              assert.equal(true, Array.isArray(data.response.firewall_sections));
              assert.equal('object', typeof data.response.advanced_config);
              assert.equal('TIER0', data.response.router_type);
              assert.equal(8, data.response.preferred_edge_cluster_member_index);
              assert.equal('ACTIVE_ACTIVE', data.response.high_availability_mode);
              assert.equal('string', data.response.edge_cluster_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('NON_PREEMPTIVE', data.response.failover_mode);
              assert.equal(true, Array.isArray(data.response.firewall_sections));
              assert.equal('object', typeof data.response.advanced_config);
              assert.equal('TIER0', data.response.router_type);
              assert.equal(1, data.response.preferred_edge_cluster_member_index);
              assert.equal('ACTIVE_STANDBY', data.response.high_availability_mode);
              assert.equal('string', data.response.edge_cluster_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLogicalRouterslogicalRouterId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLogicalRouterslogicalRouterId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdroutingadvertisementrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdroutingadvertisementrules('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.rules));
              assert.equal('string', data.response.logical_router_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingadvertisementrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingadvertisementrules('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.rules));
              assert.equal('string', data.response.logical_router_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingroutingTableformatcsv - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingroutingTableformatcsv('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.file_name);
              assert.equal(10, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postDhcprelays - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postDhcprelays('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.dhcp_relay_profile_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcprelays - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcprelays('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdroutingipPrefixListsid - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdroutingipPrefixListsid('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.prefixes));
              assert.equal('string', data.response.logical_router_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingipPrefixListsid - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingipPrefixListsid('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.prefixes));
              assert.equal('string', data.response.logical_router_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLogicalRouterslogicalRouterIdroutingipPrefixListsid - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLogicalRouterslogicalRouterIdroutingipPrefixListsid('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterPortslogicalRouterPortIdstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterPortslogicalRouterPortIdstatistics('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.per_node_statistics));
              assert.equal('string', data.response.logical_router_port_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdroutingrouteMapsid - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdroutingrouteMapsid('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, Array.isArray(data.response.sequences));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingrouteMapsid - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingrouteMapsid('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, Array.isArray(data.response.sequences));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLogicalRouterslogicalRouterIdroutingrouteMapsid - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLogicalRouterslogicalRouterIdroutingrouteMapsid('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdroutingbgp - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdroutingbgp('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(9, data.response.as_number);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, data.response.graceful_restart);
              assert.equal('string', data.response.as_num);
              assert.equal(false, data.response.enabled);
              assert.equal(true, Array.isArray(data.response.route_aggregation));
              assert.equal(true, data.response.ecmp);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingbgp - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingbgp('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(4, data.response.as_number);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(false, data.response.graceful_restart);
              assert.equal('string', data.response.as_num);
              assert.equal(true, data.response.enabled);
              assert.equal(true, Array.isArray(data.response.route_aggregation));
              assert.equal(true, data.response.ecmp);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdrouting - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdrouting('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(10, data.response.forwarding_up_timer);
              assert.equal('string', data.response.logical_router_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdrouting - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdrouting('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(3, data.response.forwarding_up_timer);
              assert.equal('string', data.response.logical_router_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdroutingadvertisement - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdroutingadvertisement('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.advertise_nsx_connected_routes);
              assert.equal(false, data.response.advertise_lb_vip);
              assert.equal(true, data.response.advertise_static_routes);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, data.response.advertise_nat_routes);
              assert.equal(true, data.response.enabled);
              assert.equal(true, data.response.advertise_lb_snat_ip);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingadvertisement - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingadvertisement('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.advertise_nsx_connected_routes);
              assert.equal(false, data.response.advertise_lb_vip);
              assert.equal(false, data.response.advertise_static_routes);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(false, data.response.advertise_nat_routes);
              assert.equal(false, data.response.enabled);
              assert.equal(true, data.response.advertise_lb_snat_ip);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putServicesserviceId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putServicesserviceId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('DhcpRelayService', data.response.resource_type);
              assert.equal('string', data.response.dhcp_relay_profile_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getServicesserviceId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getServicesserviceId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('DhcpRelayService', data.response.resource_type);
              assert.equal('string', data.response.dhcp_relay_profile_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteServicesserviceId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteServicesserviceId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdroutingbgpneighborsid - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdroutingbgpneighborsid('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.filter_in_ipprefixlist_id);
              assert.equal('string', data.response.neighbor_address);
              assert.equal(3, data.response.remote_as);
              assert.equal('string', data.response.filter_out_ipprefixlist_id);
              assert.equal(180, data.response.hold_down_timer);
              assert.equal(1, data.response.maximum_hop_limit);
              assert.equal(true, data.response.enabled);
              assert.equal('object', typeof data.response.bfd_config);
              assert.equal('string', data.response.logical_router_id);
              assert.equal('string', data.response.remote_as_num);
              assert.equal('string', data.response.filter_out_routemap_id);
              assert.equal('string', data.response.filter_in_routemap_id);
              assert.equal(60, data.response.keep_alive_timer);
              assert.equal('string', data.response.password);
              assert.equal('string', data.response.source_address);
              assert.equal(true, Array.isArray(data.response.source_addresses));
              assert.equal(false, data.response.enable_bfd);
              assert.equal(true, Array.isArray(data.response.address_families));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLogicalRouterslogicalRouterIdroutingbgpneighborsid - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLogicalRouterslogicalRouterIdroutingbgpneighborsid('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.filter_in_ipprefixlist_id);
              assert.equal('string', data.response.neighbor_address);
              assert.equal(4, data.response.remote_as);
              assert.equal('string', data.response.filter_out_ipprefixlist_id);
              assert.equal(180, data.response.hold_down_timer);
              assert.equal(1, data.response.maximum_hop_limit);
              assert.equal(true, data.response.enabled);
              assert.equal('object', typeof data.response.bfd_config);
              assert.equal('string', data.response.logical_router_id);
              assert.equal('string', data.response.remote_as_num);
              assert.equal('string', data.response.filter_out_routemap_id);
              assert.equal('string', data.response.filter_in_routemap_id);
              assert.equal(60, data.response.keep_alive_timer);
              assert.equal('string', data.response.password);
              assert.equal('string', data.response.source_address);
              assert.equal(true, Array.isArray(data.response.source_addresses));
              assert.equal(false, data.response.enable_bfd);
              assert.equal(true, Array.isArray(data.response.address_families));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingbgpneighborsid - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingbgpneighborsid('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.filter_in_ipprefixlist_id);
              assert.equal('string', data.response.neighbor_address);
              assert.equal(3, data.response.remote_as);
              assert.equal('string', data.response.filter_out_ipprefixlist_id);
              assert.equal(180, data.response.hold_down_timer);
              assert.equal(1, data.response.maximum_hop_limit);
              assert.equal(true, data.response.enabled);
              assert.equal('object', typeof data.response.bfd_config);
              assert.equal('string', data.response.logical_router_id);
              assert.equal('string', data.response.remote_as_num);
              assert.equal('string', data.response.filter_out_routemap_id);
              assert.equal('string', data.response.filter_in_routemap_id);
              assert.equal(60, data.response.keep_alive_timer);
              assert.equal('string', data.response.password);
              assert.equal('string', data.response.source_address);
              assert.equal(true, Array.isArray(data.response.source_addresses));
              assert.equal(false, data.response.enable_bfd);
              assert.equal(true, Array.isArray(data.response.address_families));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLogicalRouterslogicalRouterIdroutingbgpneighborsid - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLogicalRouterslogicalRouterIdroutingbgpneighborsid('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdroutingbfdConfig - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdroutingbfdConfig('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(1000, data.response.receive_interval);
              assert.equal(3, data.response.declare_dead_multiple);
              assert.equal(false, data.response.enabled);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(1000, data.response.transmit_interval);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdroutingbfdConfig - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdroutingbfdConfig('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(1000, data.response.receive_interval);
              assert.equal(3, data.response.declare_dead_multiple);
              assert.equal(true, data.response.enabled);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(1000, data.response.transmit_interval);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putServiceProfilesserviceProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putServiceProfilesserviceProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('DhcpRelayProfile', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.server_addresses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getServiceProfilesserviceProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getServiceProfilesserviceProfileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('DhcpRelayProfile', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.server_addresses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteServiceProfilesserviceProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteServiceProfilesserviceProfileId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterPortslogicalRouterPortIdarpTableformatcsv - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterPortslogicalRouterPortIdarpTableformatcsv('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.file_name);
              assert.equal(6, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterPortslogicalRouterPortId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterPortslogicalRouterPortId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LogicalRouterLinkPortOnTIER1', data.response.resource_type);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, Array.isArray(data.response.service_bindings));
              assert.equal('string', data.response.mac_address);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterPortslogicalRouterPortId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterPortslogicalRouterPortId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LogicalRouterLinkPortOnTIER1', data.response.resource_type);
              assert.equal('string', data.response.logical_router_id);
              assert.equal(true, Array.isArray(data.response.service_bindings));
              assert.equal('string', data.response.mac_address);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLogicalRouterPortslogicalRouterPortId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLogicalRouterPortslogicalRouterPortId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLogicalRouterslogicalRouterIdnatrulesruleId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLogicalRouterslogicalRouterIdnatrulesruleId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.match_destination_network);
              assert.equal('string', data.response.translated_network);
              assert.equal(1024, data.response.rule_priority);
              assert.equal('object', typeof data.response.match_service);
              assert.equal(true, data.response.enabled);
              assert.equal('string', data.response.logical_router_id);
              assert.equal('string', data.response.translated_ports);
              assert.equal('SNAT', data.response.action);
              assert.equal(true, data.response.logging);
              assert.equal(true, data.response.nat_pass);
              assert.equal('string', data.response.match_source_network);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdnatrulesruleId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdnatrulesruleId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.match_destination_network);
              assert.equal('string', data.response.translated_network);
              assert.equal(1024, data.response.rule_priority);
              assert.equal('object', typeof data.response.match_service);
              assert.equal(true, data.response.enabled);
              assert.equal('string', data.response.logical_router_id);
              assert.equal('string', data.response.translated_ports);
              assert.equal('NO_NAT', data.response.action);
              assert.equal(true, data.response.logging);
              assert.equal(true, data.response.nat_pass);
              assert.equal('string', data.response.match_source_network);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLogicalRouterslogicalRouterIdnatrulesruleId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLogicalRouterslogicalRouterIdnatrulesruleId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLogicalRouterslogicalRouterIdnatrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLogicalRouterslogicalRouterIdnatrules('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.match_destination_network);
              assert.equal('string', data.response.translated_network);
              assert.equal(1024, data.response.rule_priority);
              assert.equal('object', typeof data.response.match_service);
              assert.equal(true, data.response.enabled);
              assert.equal('string', data.response.logical_router_id);
              assert.equal('string', data.response.translated_ports);
              assert.equal('NO_NAT', data.response.action);
              assert.equal(true, data.response.logging);
              assert.equal(true, data.response.nat_pass);
              assert.equal('string', data.response.match_source_network);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLogicalRouterslogicalRouterIdnatrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLogicalRouterslogicalRouterIdnatrules('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusternodesnodeIdnetworkinterfaces - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusternodesnodeIdnetworkinterfaces('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessyslogexporters - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessyslogexporters('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(514, data.response.port);
              assert.equal(true, Array.isArray(data.response.facilities));
              assert.equal(true, Array.isArray(data.response.msgids));
              assert.equal('UDP', data.response.protocol);
              assert.equal('string', data.response.exporter_name);
              assert.equal('DEBUG', data.response.level);
              assert.equal(true, Array.isArray(data.response.structured_data));
              assert.equal('string', data.response.tls_ca_pem);
              assert.equal('string', data.response.server);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicessyslogexporters - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicessyslogexporters((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicescmInventoryactionstop - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicescmInventoryactionstop((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(5, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodenetwork - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodenetwork((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusternodesnodeIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusternodesnodeIdstatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.system_status);
              assert.equal('object', typeof data.response.mgmt_cluster_status);
              assert.equal('string', data.response.version);
              assert.equal('object', typeof data.response.control_cluster_status);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesmgmtPlaneBusactionrestart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesmgmtPlaneBusactionrestart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(5, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodenetworknameServers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodenetworknameServers('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.name_servers));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodenetworknameServers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodenetworknameServers((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.name_servers));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postClusternodes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postClusternodes('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.manager_role);
              assert.equal('object', typeof data.response.controller_role);
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.appliance_mgmt_listen_addr);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusternodes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusternodes('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodefileStorefileNameactioncopyToRemoteFile - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodefileStorefileNameactioncopyToRemoteFile('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodefileStorefileNamedata - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodefileStorefileNamedata('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response.created_epoch_ms);
              assert.equal(6, data.response.modified_epoch_ms);
              assert.equal('string', data.response.name);
              assert.equal(8, data.response.size);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodefileStorefileNamedata - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.getNodefileStorefileNamedata('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesnsxUpgradeAgentactionrestart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesnsxUpgradeAgentactionrestart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(4, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicessearchstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicessearchstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(10, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesntpactionrestart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesntpactionrestart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(4, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesmanageractionrestart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesmanageractionrestart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(1, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodeservicessnmp - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodeservicessnmp('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
              assert.equal('object', typeof data.response.service_properties);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicessnmp - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicessnmp((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusterrestorestatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusterrestorestatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.status);
              assert.equal('object', typeof data.response.step);
              assert.equal(true, Array.isArray(data.response.endpoints));
              assert.equal(4, data.response.total_steps);
              assert.equal(6, data.response.restore_start_time);
              assert.equal(1, data.response.restore_end_time);
              assert.equal(3, data.response.backup_timestamp);
              assert.equal('string', data.response.id);
              assert.equal(true, Array.isArray(data.response.instructions));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTrustManagement - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTrustManagement((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.supported_algorithms));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusterbackupshistory - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusterbackupshistory((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.inventory_backup_statuses));
              assert.equal(true, Array.isArray(data.response.cluster_backup_statuses));
              assert.equal(true, Array.isArray(data.response.node_backup_statuses));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesliagentactionstop - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesliagentactionstop((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(6, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodeservicesntp - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodeservicesntp('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
              assert.equal('object', typeof data.response.service_properties);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesntp - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesntp((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
              assert.equal('object', typeof data.response.service_properties);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeserviceshttpactionstop - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeserviceshttpactionstop((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesnsxUpgradeAgentactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesnsxUpgradeAgentactionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(9, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessshactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessshactionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(1, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesliagent - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesliagent((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTrustManagementcrlsactionimport - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postTrustManagementcrlsactionimport('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusterrestorebackuptimestamps - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusterrestorebackuptimestamps('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesmgmtPlaneBusactionstop - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesmgmtPlaneBusactionstop((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(5, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postAdministrationsupportBundlesactioncollect - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postAdministrationsupportBundlesactioncollect('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.request_properties);
              assert.equal(true, Array.isArray(data.response.remaining_nodes));
              assert.equal(true, Array.isArray(data.response.success_nodes));
              assert.equal(true, Array.isArray(data.response.failed_nodes));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessnmpactionstop - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessnmpactionstop((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(3, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postClusterrestoreactionretry - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postClusterrestoreactionretry((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.status);
              assert.equal('object', typeof data.response.step);
              assert.equal(true, Array.isArray(data.response.endpoints));
              assert.equal(6, data.response.total_steps);
              assert.equal(5, data.response.restore_start_time);
              assert.equal(8, data.response.restore_end_time);
              assert.equal(8, data.response.backup_timestamp);
              assert.equal('string', data.response.id);
              assert.equal(true, Array.isArray(data.response.instructions));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesinstallUpgradeactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesinstallUpgradeactionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(9, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeserviceshttpactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeserviceshttpactionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(5, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeactionrestart - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeactionrestart((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodenetworkinterfacesinterfaceId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodenetworkinterfacesinterfaceId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.physical_address);
              assert.equal('string', data.response.broadcast_address);
              assert.equal('up', data.response.link_status);
              assert.equal('string', data.response.default_gateway);
              assert.equal('string', data.response.bond_primary);
              assert.equal(true, Array.isArray(data.response.bond_slaves));
              assert.equal(true, Array.isArray(data.response.ip_addresses));
              assert.equal('down', data.response.admin_status);
              assert.equal('string', data.response.interface_id);
              assert.equal('ACTIVE_BACKUP', data.response.bond_mode);
              assert.equal('debug', data.response.plane);
              assert.equal('dhcp', data.response.ip_configuration);
              assert.equal(10, data.response.mtu);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodenetworkinterfacesinterfaceId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodenetworkinterfacesinterfaceId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.physical_address);
              assert.equal('string', data.response.broadcast_address);
              assert.equal('down', data.response.link_status);
              assert.equal('string', data.response.default_gateway);
              assert.equal('string', data.response.bond_primary);
              assert.equal(true, Array.isArray(data.response.bond_slaves));
              assert.equal(true, Array.isArray(data.response.ip_addresses));
              assert.equal('up', data.response.admin_status);
              assert.equal('string', data.response.interface_id);
              assert.equal('802_3AD', data.response.bond_mode);
              assert.equal('mgmt', data.response.plane);
              assert.equal('not configured', data.response.ip_configuration);
              assert.equal(10, data.response.mtu);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodeservicesssh - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodeservicesssh('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
              assert.equal('object', typeof data.response.service_properties);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesssh - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesssh((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
              assert.equal('object', typeof data.response.service_properties);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNoderabbitmqManagementPort - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNoderabbitmqManagementPort((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteNoderabbitmqManagementPort - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteNoderabbitmqManagementPort((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNoderabbitmqManagementPort - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNoderabbitmqManagementPort((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, data.response.enabled);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteNodetaskstaskId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteNodetaskstaskId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodetaskstaskId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodetaskstaskId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('canceling', data.response.status);
              assert.equal(10, data.response.end_time);
              assert.equal(true, data.response.async_response_available);
              assert.equal('string', data.response.description);
              assert.equal(10, data.response.start_time);
              assert.equal(false, data.response.cancelable);
              assert.equal('string', data.response.request_method);
              assert.equal('object', typeof data.response.details);
              assert.equal(3, data.response.progress);
              assert.equal('string', data.response.message);
              assert.equal('string', data.response.user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.request_uri);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postClusteractionsummarizeInventoryToRemote - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postClusteractionsummarizeInventoryToRemote((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodetaskstaskIdresponse - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.getNodetaskstaskIdresponse('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesmgmtPlaneBusstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesmgmtPlaneBusstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(8, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postClusteractionbackupToRemote - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postClusteractionbackupToRemote((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusterbackupsstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusterbackupsstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('BACKUP_CREATING_NODE_BACKUP', data.response.current_step);
              assert.equal('string', data.response.backup_id);
              assert.equal('string', data.response.current_step_message);
              assert.equal(3, data.response.end_time);
              assert.equal('BACKUP', data.response.operation_type);
              assert.equal(3, data.response.start_time);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTrustManagementcsrscsrIdactionimport - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postTrustManagementcsrscsrIdactionimport('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesntpactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesntpactionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(7, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusterstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusterstatus('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.mgmt_cluster_status);
              assert.equal('object', typeof data.response.control_cluster_status);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesmanageractionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesmanageractionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(5, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicescmInventorystatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicescmInventorystatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(9, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodeusersuserid - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodeusersuserid('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.username);
              assert.equal('string', data.response.password);
              assert.equal(2, data.response.userid);
              assert.equal('string', data.response.old_password);
              assert.equal('string', data.response.full_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeusersuserid - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeusersuserid('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.username);
              assert.equal('string', data.response.password);
              assert.equal(5, data.response.userid);
              assert.equal('string', data.response.old_password);
              assert.equal('string', data.response.full_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicescmInventoryactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicescmInventoryactionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(5, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesnsxMessageBusactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesnsxMessageBusactionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(1, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusternodesnodeIdnetworkinterfacesinterfaceIdstats - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusternodesnodeIdnetworkinterfacesinterfaceIdstats('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response.tx_dropped);
              assert.equal(3, data.response.rx_packets);
              assert.equal(3, data.response.tx_carrier);
              assert.equal(3, data.response.rx_bytes);
              assert.equal(4, data.response.tx_errors);
              assert.equal('string', data.response.interface_id);
              assert.equal(2, data.response.tx_colls);
              assert.equal(9, data.response.rx_frame);
              assert.equal(1, data.response.rx_errors);
              assert.equal(4, data.response.tx_bytes);
              assert.equal(9, data.response.rx_dropped);
              assert.equal(3, data.response.tx_packets);
              assert.equal('realtime', data.response.source);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postClusternodesactionrevokeMissingNodes - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postClusternodesactionrevokeMissingNodes('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putConfigsmanagement - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putConfigsmanagement('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(false, data.response.publish_fqdns);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getConfigsmanagement - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getConfigsmanagement((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response.publish_fqdns);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesntpstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesntpstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(9, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusterrestoreinstructionResources - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusterrestoreinstructionResources('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTrustManagementcertificates - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTrustManagementcertificates('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTrustManagementcertificatescertId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTrustManagementcertificatescertId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.used_by));
              assert.equal(true, Array.isArray(data.response.details));
              assert.equal('string', data.response.pem_encoded);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteTrustManagementcertificatescertId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteTrustManagementcertificatescertId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodenetworksearchDomains - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodenetworksearchDomains('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.search_domains));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodenetworksearchDomains - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodenetworksearchDomains((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.search_domains));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesnodeMgmtactionrestart - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesnodeMgmtactionrestart((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessshactionrestart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessshactionrestart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(10, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicessyslogexportersexporterName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicessyslogexportersexporterName('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(514, data.response.port);
              assert.equal(true, Array.isArray(data.response.facilities));
              assert.equal(true, Array.isArray(data.response.msgids));
              assert.equal('LI', data.response.protocol);
              assert.equal('string', data.response.exporter_name);
              assert.equal('WARNING', data.response.level);
              assert.equal(true, Array.isArray(data.response.structured_data));
              assert.equal('string', data.response.tls_ca_pem);
              assert.equal('string', data.response.server);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteNodeservicessyslogexportersexporterName - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteNodeservicessyslogexportersexporterName('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusternodesnodeIdnetworkinterfacesinterfaceId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusternodesnodeIdnetworkinterfacesinterfaceId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('cached', data.response.source);
              assert.equal('UP', data.response.admin_status);
              assert.equal('UP', data.response.link_status);
              assert.equal(true, Array.isArray(data.response.interface_alias));
              assert.equal(false, data.response.ens_enabled);
              assert.equal('VIRTUAL', data.response.interface_type);
              assert.equal('string', data.response.interface_id);
              assert.equal('string', data.response.connected_switch);
              assert.equal(false, data.response.ens_capable);
              assert.equal(10, data.response.mtu);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessnmpactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessnmpactionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(5, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessnmpactionrestart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessnmpactionrestart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(1, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodefileStorefileNamethumbprint - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodefileStorefileNamethumbprint('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.sha256);
              assert.equal('string', data.response.name);
              assert.equal('string', data.response.sha1);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesliagentstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesliagentstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(8, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postClusterrestoreactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postClusterrestoreactionstart('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.status);
              assert.equal('object', typeof data.response.step);
              assert.equal(true, Array.isArray(data.response.endpoints));
              assert.equal(5, data.response.total_steps);
              assert.equal(9, data.response.restore_start_time);
              assert.equal(7, data.response.restore_end_time);
              assert.equal(8, data.response.backup_timestamp);
              assert.equal('string', data.response.id);
              assert.equal(true, Array.isArray(data.response.instructions));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesnsxMessageBusactionstop - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesnsxMessageBusactionstop((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(7, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicessnmpstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicessnmpstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(4, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodesupportBundle - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.getNodesupportBundle('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicessearch - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicessearch((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postClusterrestoreactioncancel - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postClusterrestoreactioncancel((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.status);
              assert.equal('object', typeof data.response.step);
              assert.equal(true, Array.isArray(data.response.endpoints));
              assert.equal(6, data.response.total_steps);
              assert.equal(3, data.response.restore_start_time);
              assert.equal(9, data.response.restore_end_time);
              assert.equal(3, data.response.backup_timestamp);
              assert.equal('string', data.response.id);
              assert.equal(true, Array.isArray(data.response.instructions));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTrustManagementcsrscsrIdpemFile - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.getTrustManagementcsrscsrIdpemFile('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postClusterrestoreactionadvance - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postClusterrestoreactionadvance('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.status);
              assert.equal('object', typeof data.response.step);
              assert.equal(true, Array.isArray(data.response.endpoints));
              assert.equal(9, data.response.total_steps);
              assert.equal(10, data.response.restore_start_time);
              assert.equal(6, data.response.restore_end_time);
              assert.equal(9, data.response.backup_timestamp);
              assert.equal('string', data.response.id);
              assert.equal(true, Array.isArray(data.response.instructions));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicessshstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicessshstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(3, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesnsxMessageBusactionrestart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesnsxMessageBusactionrestart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(3, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putClusterrestoreconfig - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putClusterrestoreconfig('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.remote_file_server);
              assert.equal('string', data.response.passphrase);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusterrestoreconfig - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusterrestoreconfig((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.remote_file_server);
              assert.equal('string', data.response.passphrase);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicescmInventory - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicescmInventory((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postClusterrestoreactionsuspend - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postClusterrestoreactionsuspend((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.status);
              assert.equal('object', typeof data.response.step);
              assert.equal(true, Array.isArray(data.response.endpoints));
              assert.equal(1, data.response.total_steps);
              assert.equal(10, data.response.restore_start_time);
              assert.equal(2, data.response.restore_end_time);
              assert.equal(5, data.response.backup_timestamp);
              assert.equal('string', data.response.id);
              assert.equal(true, Array.isArray(data.response.instructions));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodetaskstaskIdactioncancel - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodetaskstaskIdactioncancel('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteTrustManagementprincipalIdentitiesprincipalIdentityId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteTrustManagementprincipalIdentitiesprincipalIdentityId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodefileStorefileName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodefileStorefileName('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response.created_epoch_ms);
              assert.equal(9, data.response.modified_epoch_ms);
              assert.equal('string', data.response.name);
              assert.equal(8, data.response.size);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteNodefileStorefileName - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteNodefileStorefileName('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodefileStorefileName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodefileStorefileName('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response.created_epoch_ms);
              assert.equal(7, data.response.modified_epoch_ms);
              assert.equal('string', data.response.name);
              assert.equal(6, data.response.size);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicescmInventoryactionrestart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicescmInventoryactionrestart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(6, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeusersuseridsshKeysactionaddSshKey - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeusersuseridsshKeysactionaddSshKey('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessshactionremoveHostFingerprint - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessshactionremoveHostFingerprint('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodetasks - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodetasks('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesmgmtPlaneBusactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesmgmtPlaneBusactionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(7, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesnsxMessageBusstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesnsxMessageBusstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(10, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesnodeMgmtstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesnodeMgmtstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(6, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteClusternodesnodeId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteClusternodesnodeId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusternodesnodeId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusternodesnodeId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.manager_role);
              assert.equal('object', typeof data.response.controller_role);
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.appliance_mgmt_listen_addr);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesntpactionstop - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesntpactionstop((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(8, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessyslogactionrestart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessyslogactionrestart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(2, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodeservicesmanager - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodeservicesmanager('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
              assert.equal('object', typeof data.response.service_properties);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesmanager - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesmanager((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
              assert.equal('object', typeof data.response.service_properties);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeserviceshttpactionrestart - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeserviceshttpactionrestart((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeaaaprovidersvidmstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeaaaprovidersvidmstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.runtime_state);
              assert.equal(false, data.response.vidm_enable);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesmanageractionresetManagerLoggingLevels - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesmanageractionresetManagerLoggingLevels((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeusers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeusers((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusternodesstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusternodesstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.management_cluster));
              assert.equal(true, Array.isArray(data.response.controller_cluster));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesnsxUpgradeAgentstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesnsxUpgradeAgentstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(7, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodefileStorefileNameactioncopyFromRemoteFile - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodefileStorefileNameactioncopyFromRemoteFile('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response.created_epoch_ms);
              assert.equal(5, data.response.modified_epoch_ms);
              assert.equal('string', data.response.name);
              assert.equal(5, data.response.size);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesnodeMgmt - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesnodeMgmt((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodeservicesinstallUpgrade - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodeservicesinstallUpgrade('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
              assert.equal('object', typeof data.response.service_properties);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesinstallUpgrade - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesinstallUpgrade((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
              assert.equal('object', typeof data.response.service_properties);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeprocesses - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeprocesses((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteNodenetworkroutesrouteId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteNodenetworkroutesrouteId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodenetworkroutesrouteId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodenetworkroutesrouteId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.src);
              assert.equal('string', data.response.from_address);
              assert.equal('boot', data.response.proto);
              assert.equal('unreachable', data.response.route_type);
              assert.equal('string', data.response.metric);
              assert.equal('string', data.response.destination);
              assert.equal('string', data.response.interface_id);
              assert.equal('string', data.response.route_id);
              assert.equal('string', data.response.netmask);
              assert.equal('string', data.response.scope);
              assert.equal('string', data.response.gateway);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodefileStore - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodefileStore((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTrustManagementcsrs - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postTrustManagementcsrs('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(10, data.response.key_size);
              assert.equal('string', data.response.pem_encoded);
              assert.equal('DSA', data.response.algorithm);
              assert.equal('object', typeof data.response.subject);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTrustManagementcsrs - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTrustManagementcsrs('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesnsxUpgradeAgent - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesnsxUpgradeAgent((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodehardeningPolicymandatoryAccessControlreport - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.getNodehardeningPolicymandatoryAccessControlreport((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesinstallUpgradestatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesinstallUpgradestatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(4, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCluster - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCluster((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response.control_cluster_changes_allowed);
              assert.equal(true, data.response.mgmt_cluster_changes_allowed);
              assert.equal('string', data.response.cluster_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodefileStoreactioncreateRemoteDirectory - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodefileStoreactioncreateRemoteDirectory('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessyslogactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessyslogactionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(6, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicessyslog - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicessyslog((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putClusterbackupsconfig - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putClusterbackupsconfig('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.backup_schedule);
              assert.equal('object', typeof data.response.remote_file_server);
              assert.equal('string', data.response.passphrase);
              assert.equal(true, data.response.backup_enabled);
              assert.equal(300, data.response.inventory_summary_interval);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusterbackupsconfig - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusterbackupsconfig((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.backup_schedule);
              assert.equal('object', typeof data.response.remote_file_server);
              assert.equal('string', data.response.passphrase);
              assert.equal(true, data.response.backup_enabled);
              assert.equal(300, data.response.inventory_summary_interval);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putTrustManagementcrlscrlId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putTrustManagementcrlscrlId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.details);
              assert.equal('string', data.response.pem_encoded);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTrustManagementcrlscrlId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTrustManagementcrlscrlId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.details);
              assert.equal('string', data.response.pem_encoded);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteTrustManagementcrlscrlId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteTrustManagementcrlscrlId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTrustManagementcsrscsrIdactionselfSign - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postTrustManagementcsrscsrIdactionselfSign('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.used_by));
              assert.equal(true, Array.isArray(data.response.details));
              assert.equal('string', data.response.pem_encoded);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeusersuseridsshKeysactionremoveSshKey - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeusersuseridsshKeysactionremoveSshKey('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodenetworkinterfacesinterfaceIdstats - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodenetworkinterfacesinterfaceIdstats('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response.tx_dropped);
              assert.equal(10, data.response.rx_packets);
              assert.equal(4, data.response.tx_carrier);
              assert.equal(5, data.response.rx_bytes);
              assert.equal(9, data.response.tx_errors);
              assert.equal('string', data.response.interface_id);
              assert.equal(7, data.response.tx_colls);
              assert.equal(8, data.response.rx_frame);
              assert.equal(6, data.response.rx_errors);
              assert.equal(6, data.response.tx_bytes);
              assert.equal(2, data.response.rx_dropped);
              assert.equal(10, data.response.tx_packets);
              assert.equal('cached', data.response.source);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessshactionstop - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessshactionstop((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(8, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicessyslogstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicessyslogstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(5, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTrustManagementcertificatesactionimport - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postTrustManagementcertificatesactionimport('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessyslogactionstop - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessyslogactionstop((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(1, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTrustManagementprincipalIdentities - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postTrustManagementprincipalIdentities('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('read_only_api_users', data.response.permission_group);
              assert.equal('string', data.response.node_id);
              assert.equal('string', data.response.name);
              assert.equal(true, data.response.is_protected);
              assert.equal('string', data.response.certificate_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTrustManagementprincipalIdentities - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTrustManagementprincipalIdentities((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesmanageractionstop - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesmanageractionstop((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(5, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTrustManagementcrls - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTrustManagementcrls('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodeaaaprovidersvidm - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodeaaaprovidersvidm('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.host_name);
              assert.equal('string', data.response.client_id);
              assert.equal('string', data.response.thumbprint);
              assert.equal('string', data.response.client_secret);
              assert.equal('string', data.response.node_host_name);
              assert.equal(false, data.response.vidm_enable);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeaaaprovidersvidm - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeaaaprovidersvidm((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.host_name);
              assert.equal('string', data.response.client_id);
              assert.equal('string', data.response.thumbprint);
              assert.equal('string', data.response.client_secret);
              assert.equal('string', data.response.node_host_name);
              assert.equal(false, data.response.vidm_enable);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservices - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservices((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteTrustManagementcsrscsrId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteTrustManagementcsrscsrId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTrustManagementcsrscsrId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTrustManagementcsrscsrId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(5, data.response.key_size);
              assert.equal('string', data.response.pem_encoded);
              assert.equal('RSA', data.response.algorithm);
              assert.equal('object', typeof data.response.subject);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeactionshutdown - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeactionshutdown((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeserviceshttpstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeserviceshttpstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(1, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodenetworkinterfaces - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodenetworkinterfaces((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeserviceshttpactionapplyCertificate - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeserviceshttpactionapplyCertificate('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessearchactionstop - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessearchactionstop((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(8, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeusersuseridsshKeys - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeusersuseridsshKeys('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessearchactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessearchactionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(1, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicessearchactionrestart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicessearchactionrestart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(1, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesinstallUpgradeactionrestart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesinstallUpgradeactionrestart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(7, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesmanagerstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesmanagerstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(2, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesliagentactionrestart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesliagentactionrestart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('running', data.response.runtime_state);
              assert.equal('running', data.response.monitor_runtime_state);
              assert.equal(2, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesnsxMessageBus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesnsxMessageBus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesinstallUpgradeactionstop - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesinstallUpgradeactionstop((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(5, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesnsxUpgradeAgentactionstop - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesnsxUpgradeAgentactionstop((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(5, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodehardeningPolicymandatoryAccessControl - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodehardeningPolicymandatoryAccessControl('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('ENABLED_PENDING_REBOOT', data.response.status);
              assert.equal(false, data.response.enabled);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodehardeningPolicymandatoryAccessControl - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodehardeningPolicymandatoryAccessControl((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('DISABLED', data.response.status);
              assert.equal(true, data.response.enabled);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNode - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNode('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.kernel_version);
              assert.equal(9, data.response.system_time);
              assert.equal('string', data.response.node_uuid);
              assert.equal('object', typeof data.response.motd);
              assert.equal(4, data.response.cli_timeout);
              assert.equal('string', data.response.bios_uuid);
              assert.equal('string', data.response.timezone);
              assert.equal('string', data.response.hostname);
              assert.equal('string', data.response.node_version);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNode - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNode((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.kernel_version);
              assert.equal(3, data.response.system_time);
              assert.equal('string', data.response.node_uuid);
              assert.equal('object', typeof data.response.motd);
              assert.equal(2, data.response.cli_timeout);
              assert.equal('string', data.response.bios_uuid);
              assert.equal('string', data.response.timezone);
              assert.equal('string', data.response.hostname);
              assert.equal('string', data.response.node_version);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodeserviceshttp - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodeserviceshttp('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
              assert.equal('object', typeof data.response.service_properties);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeserviceshttp - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeserviceshttp((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
              assert.equal('object', typeof data.response.service_properties);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeprocessesprocessId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeprocessesprocessId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response.mem_used);
              assert.equal(3, data.response.cpu_time);
              assert.equal(4, data.response.ppid);
              assert.equal(6, data.response.start_time);
              assert.equal('string', data.response.process_name);
              assert.equal(8, data.response.pid);
              assert.equal(4, data.response.uptime);
              assert.equal(1, data.response.mem_resident);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodeservicesliagentactionstart - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodeservicesliagentactionstart((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.pids));
              assert.equal('stopped', data.response.runtime_state);
              assert.equal('stopped', data.response.monitor_runtime_state);
              assert.equal(6, data.response.monitor_pid);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeservicesmgmtPlaneBus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeservicesmgmtPlaneBus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.service_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNodenetworkroutes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNodenetworkroutes('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.src);
              assert.equal('string', data.response.from_address);
              assert.equal('boot', data.response.proto);
              assert.equal('unreachable', data.response.route_type);
              assert.equal('string', data.response.metric);
              assert.equal('string', data.response.destination);
              assert.equal('string', data.response.interface_id);
              assert.equal('string', data.response.route_id);
              assert.equal('string', data.response.netmask);
              assert.equal('string', data.response.scope);
              assert.equal('string', data.response.gateway);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodenetworkroutes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodenetworkroutes((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradenodesSummary - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradenodesSummary((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postUpgradeupgradeUnitGroups - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postUpgradeupgradeUnitGroups('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.enabled);
              assert.equal(5, data.response.upgrade_unit_count);
              assert.equal('string', data.response.type);
              assert.equal(true, Array.isArray(data.response.upgrade_units));
              assert.equal(true, Array.isArray(data.response.extended_configuration));
              assert.equal(true, data.response.parallel);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeupgradeUnitGroups - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeupgradeUnitGroups('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putUpgradeupgradeUnitGroupsgroupId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putUpgradeupgradeUnitGroupsgroupId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.enabled);
              assert.equal(9, data.response.upgrade_unit_count);
              assert.equal('string', data.response.type);
              assert.equal(true, Array.isArray(data.response.upgrade_units));
              assert.equal(true, Array.isArray(data.response.extended_configuration));
              assert.equal(true, data.response.parallel);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteUpgradeupgradeUnitGroupsgroupId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteUpgradeupgradeUnitGroupsgroupId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeupgradeUnitGroupsgroupId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeupgradeUnitGroupsgroupId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.enabled);
              assert.equal(2, data.response.upgrade_unit_count);
              assert.equal('string', data.response.type);
              assert.equal(true, Array.isArray(data.response.upgrade_units));
              assert.equal(true, Array.isArray(data.response.extended_configuration));
              assert.equal(true, data.response.parallel);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeversionWhitelist - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeversionWhitelist((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeupgradeUnitsStats - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeupgradeUnitsStats('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postUpgradeplanactionstart - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postUpgradeplanactionstart((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putUpgradeversionWhitelistcomponentType - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.putUpgradeversionWhitelistcomponentType('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeversionWhitelistcomponentType - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeversionWhitelistcomponentType('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.acceptable_versions));
              assert.equal('MP', data.response.component_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postUpgradeplanactionpause - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postUpgradeplanactionpause((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postUpgradeupgradeUnitGroupsgroupIdupgradeUnitupgradeUnitIdactionreorder - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postUpgradeupgradeUnitGroupsgroupIdupgradeUnitupgradeUnitIdactionreorder('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeupgradeUnitGroupsaggregateInfo - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeupgradeUnitGroupsaggregateInfo('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeupgradeUnitsupgradeUnitId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeupgradeUnitsupgradeUnitId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('object', typeof data.response.group);
              assert.equal(true, Array.isArray(data.response.warnings));
              assert.equal('string', data.response.current_version);
              assert.equal(true, Array.isArray(data.response.metadata));
              assert.equal('string', data.response.type);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.display_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postUpgradeupgradeUnitGroupsgroupIdactionreorder - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postUpgradeupgradeUnitGroupsgroupIdactionreorder('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postUpgradeplanactioncontinue - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postUpgradeplanactioncontinue('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeupgradeUnitsaggregateInfo - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeupgradeUnitsaggregateInfo('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postUpgradeupgradeUnitGroupsgroupIdactionaddUpgradeUnits - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postUpgradeupgradeUnitGroupsgroupIdactionaddUpgradeUnits('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.list));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeupgradeUnits - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeupgradeUnits('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradestatusSummary - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradestatusSummary('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.host_status);
              assert.equal('object', typeof data.response.ccp_status);
              assert.equal('object', typeof data.response.edge_status);
              assert.equal('SUCCESS', data.response.overall_upgrade_status);
              assert.equal(true, Array.isArray(data.response.component_status));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeupgradeUnitGroupsgroupIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeupgradeUnitGroupsgroupIdstatus('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postUpgradeplanactionupgradeSelectedUnits - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postUpgradeplanactionupgradeSelectedUnits('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeupgradeUnitGroupsStatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeupgradeUnitGroupsStatus('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradesummary - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradesummary((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.target_version);
              assert.equal('string', data.response.upgrade_coordinator_version);
              assert.equal('FAILED', data.response.upgrade_status);
              assert.equal(true, Array.isArray(data.response.component_target_versions));
              assert.equal('string', data.response.system_version);
              assert.equal('string', data.response.upgrade_bundle_file_name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradenodes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradenodes('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postUpgradeactionupgradeUc - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postUpgradeactionupgradeUc((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradehistory - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradehistory((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putUpgradeplancomponentTypesettings - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putUpgradeplancomponentTypesettings('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, data.response.pause_after_each_group);
              assert.equal(false, data.response.pause_on_error);
              assert.equal(true, data.response.parallel);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeplancomponentTypesettings - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeplancomponentTypesettings('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(false, data.response.pause_after_each_group);
              assert.equal(false, data.response.pause_on_error);
              assert.equal(true, data.response.parallel);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postUpgradeplanactionreset - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postUpgradeplanactionreset('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postDhcpserversserverIdipPools - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postDhcpserversserverIdipPools('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(86400, data.response.lease_time);
              assert.equal('string', data.response.gateway_ip);
              assert.equal('object', typeof data.response.options);
              assert.equal(true, Array.isArray(data.response.allocation_ranges));
              assert.equal(80, data.response.warning_threshold);
              assert.equal(100, data.response.error_threshold);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcpserversserverIdipPools - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcpserversserverIdipPools('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postDhcpserverProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postDhcpserverProfiles('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.edge_cluster_member_indexes));
              assert.equal('string', data.response.edge_cluster_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcpserverProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcpserverProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionkeyPolicieskeyPolicyIdstats - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionkeyPolicieskeyPolicyIdstats('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response.bytes_decrypted);
              assert.equal(3, data.response.bytes_dropped);
              assert.equal(1, data.response.packets_dropped);
              assert.equal(1, data.response.packets_encrypted);
              assert.equal('string', data.response.key_policy_identifier);
              assert.equal(7, data.response.bytes_encrypted);
              assert.equal(7, data.response.packets_decrypted);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLoadbalancerapplicationProfilesapplicationProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLoadbalancerapplicationProfilesapplicationProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LbFastTcpProfile', data.response.resource_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLoadbalancerapplicationProfilesapplicationProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLoadbalancerapplicationProfilesapplicationProfileId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerapplicationProfilesapplicationProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerapplicationProfilesapplicationProfileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LbHttpProfile', data.response.resource_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallsections - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallsections('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.stateful);
              assert.equal(true, data.response.is_default);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(10, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFirewallsections - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFirewallsections('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLoadbalancerserverSslProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLoadbalancerserverSslProfiles('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.session_cache_enabled);
              assert.equal(true, data.response.is_secure);
              assert.equal(true, Array.isArray(data.response.ciphers));
              assert.equal(true, Array.isArray(data.response.protocols));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerserverSslProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerserverSslProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLoadbalancerservicesserviceId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLoadbalancerservicesserviceId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.access_log_enabled);
              assert.equal('object', typeof data.response.attachment);
              assert.equal('INFO', data.response.error_log_level);
              assert.equal(true, data.response.enabled);
              assert.equal(true, Array.isArray(data.response.virtual_server_ids));
              assert.equal('SMALL', data.response.size);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLoadbalancerservicesserviceId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLoadbalancerservicesserviceId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerservicesserviceId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerservicesserviceId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.access_log_enabled);
              assert.equal('object', typeof data.response.attachment);
              assert.equal('INFO', data.response.error_log_level);
              assert.equal(true, data.response.enabled);
              assert.equal(true, Array.isArray(data.response.virtual_server_ids));
              assert.equal('SMALL', data.response.size);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLoadbalancerrulesruleId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLoadbalancerrulesruleId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('HTTP_REQUEST_REWRITE', data.response.phase);
              assert.equal(true, Array.isArray(data.response.match_conditions));
              assert.equal(true, Array.isArray(data.response.actions));
              assert.equal('ALL', data.response.match_strategy);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerrulesruleId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerrulesruleId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('HTTP_RESPONSE_REWRITE', data.response.phase);
              assert.equal(true, Array.isArray(data.response.match_conditions));
              assert.equal(true, Array.isArray(data.response.actions));
              assert.equal('ALL', data.response.match_strategy);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLoadbalancerrulesruleId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLoadbalancerrulesruleId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcpserversserverIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcpserversserverIdstatus('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.error_message);
              assert.equal('UP', data.response.service_status);
              assert.equal('string', data.response.stand_by_node);
              assert.equal('string', data.response.active_node);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postDhcpservers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postDhcpservers('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.attached_logical_port_id);
              assert.equal('object', typeof data.response.ipv4_dhcp_server);
              assert.equal('string', data.response.dhcp_profile_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcpservers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcpservers('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putDhcpserversserverIdipPoolspoolId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putDhcpserversserverIdipPoolspoolId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(86400, data.response.lease_time);
              assert.equal('string', data.response.gateway_ip);
              assert.equal('object', typeof data.response.options);
              assert.equal(true, Array.isArray(data.response.allocation_ranges));
              assert.equal(80, data.response.warning_threshold);
              assert.equal(100, data.response.error_threshold);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcpserversserverIdipPoolspoolId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcpserversserverIdipPoolspoolId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(86400, data.response.lease_time);
              assert.equal('string', data.response.gateway_ip);
              assert.equal('object', typeof data.response.options);
              assert.equal(true, Array.isArray(data.response.allocation_ranges));
              assert.equal(80, data.response.warning_threshold);
              assert.equal(100, data.response.error_threshold);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteDhcpserversserverIdipPoolspoolId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteDhcpserversserverIdipPoolspoolId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallsectionssectionIdactionreviseWithRules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallsectionssectionIdactionreviseWithRules('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.stateful);
              assert.equal(false, data.response.is_default);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(5, data.response.rule_count);
              assert.equal('LAYER2', data.response.section_type);
              assert.equal(true, Array.isArray(data.response.rules));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postDhcpserversserverIdstaticBindings - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postDhcpserversserverIdstaticBindings('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(86400, data.response.lease_time);
              assert.equal('string', data.response.gateway_ip);
              assert.equal('object', typeof data.response.options);
              assert.equal('string', data.response.ip_address);
              assert.equal('string', data.response.host_name);
              assert.equal('string', data.response.mac_address);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcpserversserverIdstaticBindings - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcpserversserverIdstaticBindings('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionsectionssectionIdrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionsectionssectionIdrules('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal('object', typeof data.response._owner);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.description);
              assert.equal('string', data.response.rule_tag);
              assert.equal('IPV4_IPV6', data.response.ip_protocol);
              assert.equal('string', data.response.key_policy_identifier);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(false, data.response.logged);
              assert.equal('string', data.response.comments);
              assert.equal(false, data.response.disabled);
              assert.equal(true, Array.isArray(data.response.sources));
              assert.equal(true, Array.isArray(data.response.services));
              assert.equal('string', data.response.key_policy_name);
              assert.equal('string', data.response.action);
              assert.equal(true, Array.isArray(data.response.destinations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionsectionssectionIdrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionsectionssectionIdrules('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putFirewallstatuscontextType - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putFirewallstatuscontextType('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('ENABLED', data.response.global_status);
              assert.equal('transport_nodes', data.response.context);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFirewallstatuscontextType - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFirewallstatuscontextType('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('DISABLED', data.response.global_status);
              assert.equal('transport_nodes', data.response.context);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionsectionssectionIdactionreviseWithRules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionsectionssectionIdactionreviseWithRules('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.is_default);
              assert.equal(5, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
              assert.equal(true, Array.isArray(data.response.rules));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLoadbalancerpoolspoolId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLoadbalancerpoolspoolId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.member_group);
              assert.equal('object', typeof data.response.snat_translation);
              assert.equal('ROUND_ROBIN', data.response.algorithm);
              assert.equal(true, Array.isArray(data.response.members));
              assert.equal('string', data.response.passive_monitor_id);
              assert.equal(6, data.response.tcp_multiplexing_number);
              assert.equal(true, Array.isArray(data.response.active_monitor_ids));
              assert.equal(false, data.response.tcp_multiplexing_enabled);
              assert.equal(1, data.response.min_active_members);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLoadbalancerpoolspoolId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLoadbalancerpoolspoolId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.member_group);
              assert.equal('object', typeof data.response.snat_translation);
              assert.equal('ROUND_ROBIN', data.response.algorithm);
              assert.equal(true, Array.isArray(data.response.members));
              assert.equal('string', data.response.passive_monitor_id);
              assert.equal(6, data.response.tcp_multiplexing_number);
              assert.equal(true, Array.isArray(data.response.active_monitor_ids));
              assert.equal(false, data.response.tcp_multiplexing_enabled);
              assert.equal(1, data.response.min_active_members);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLoadbalancerpoolspoolId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLoadbalancerpoolspoolId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerpoolspoolId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerpoolspoolId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.member_group);
              assert.equal('object', typeof data.response.snat_translation);
              assert.equal('ROUND_ROBIN', data.response.algorithm);
              assert.equal(true, Array.isArray(data.response.members));
              assert.equal('string', data.response.passive_monitor_id);
              assert.equal(6, data.response.tcp_multiplexing_number);
              assert.equal(true, Array.isArray(data.response.active_monitor_ids));
              assert.equal(true, data.response.tcp_multiplexing_enabled);
              assert.equal(1, data.response.min_active_members);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallsectionssectionIdactionupdateWithRules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallsectionssectionIdactionupdateWithRules('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.stateful);
              assert.equal(true, data.response.is_default);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(10, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
              assert.equal(true, Array.isArray(data.response.rules));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallsectionssectionIdactionlistWithRules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallsectionssectionIdactionlistWithRules('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.stateful);
              assert.equal(false, data.response.is_default);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(4, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
              assert.equal(true, Array.isArray(data.response.rules));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerservicesserviceIdpoolspoolIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerservicesserviceIdpoolspoolIdstatus('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('DOWN', data.response.status);
              assert.equal(7, data.response.last_update_timestamp);
              assert.equal('string', data.response.pool_id);
              assert.equal(true, Array.isArray(data.response.members));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('ENABLED', data.response.status);
              assert.equal('string', data.response.context);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putDhcpserversserverId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putDhcpserversserverId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.attached_logical_port_id);
              assert.equal('object', typeof data.response.ipv4_dhcp_server);
              assert.equal('string', data.response.dhcp_profile_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteDhcpserversserverId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteDhcpserversserverId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcpserversserverId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcpserversserverId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.attached_logical_port_id);
              assert.equal('object', typeof data.response.ipv4_dhcp_server);
              assert.equal('string', data.response.dhcp_profile_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallsectionssectionIdrulesactioncreateMultiple - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallsectionssectionIdrulesactioncreateMultiple('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.rules));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNetworkEncryptionkeyPolicieskeyPolicyId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNetworkEncryptionkeyPolicieskeyPolicyId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.is_default);
              assert.equal('ENC_NULL', data.response.encrypt_algorithm);
              assert.equal('MAC_ALG_AES_GCM_128', data.response.mac_algorithm);
              assert.equal('REGULAR', data.response.type);
              assert.equal('ENCRYPTION_AND_INTEGRITY', data.response.encrypt_type);
              assert.equal(2592000, data.response.rekey_frequency);
              assert.equal('string', data.response.notes);
              assert.equal(true, data.response.is_sys_default);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteNetworkEncryptionkeyPolicieskeyPolicyId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteNetworkEncryptionkeyPolicieskeyPolicyId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionkeyPolicieskeyPolicyId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionkeyPolicieskeyPolicyId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.is_default);
              assert.equal('ENC_NULL', data.response.encrypt_algorithm);
              assert.equal('MAC_ALG_AES_GCM_128', data.response.mac_algorithm);
              assert.equal('REGULAR', data.response.type);
              assert.equal('ENCRYPTION_AND_INTEGRITY', data.response.encrypt_type);
              assert.equal(2592000, data.response.rekey_frequency);
              assert.equal('string', data.response.notes);
              assert.equal(true, data.response.is_sys_default);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionsectionsactioncreateWithRules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionsectionsactioncreateWithRules('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.is_default);
              assert.equal(9, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
              assert.equal(true, Array.isArray(data.response.rules));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLoadbalancerrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLoadbalancerrules('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('HTTP_FORWARDING', data.response.phase);
              assert.equal(true, Array.isArray(data.response.match_conditions));
              assert.equal(true, Array.isArray(data.response.actions));
              assert.equal('ANY', data.response.match_strategy);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerrules('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNetworkEncryptionsectionssectionId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNetworkEncryptionsectionssectionId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.is_default);
              assert.equal(5, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionsectionssectionId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionsectionssectionId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.is_default);
              assert.equal(2, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteNetworkEncryptionsectionssectionId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteNetworkEncryptionsectionssectionId('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcpserversserverIdstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcpserversserverIdstatistics('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(8, data.response.errors);
              assert.equal(1, data.response.releases);
              assert.equal(9, data.response.informs);
              assert.equal(4, data.response.declines);
              assert.equal('string', data.response.dhcp_server_id);
              assert.equal(6, data.response.nacks);
              assert.equal(9, data.response.offers);
              assert.equal(7, data.response.discovers);
              assert.equal(5, data.response.acks);
              assert.equal(5, data.response.timestamp);
              assert.equal(7, data.response.requests);
              assert.equal(true, Array.isArray(data.response.ip_pool_stats));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLoadbalancerpools - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLoadbalancerpools('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.member_group);
              assert.equal('object', typeof data.response.snat_translation);
              assert.equal('ROUND_ROBIN', data.response.algorithm);
              assert.equal(true, Array.isArray(data.response.members));
              assert.equal('string', data.response.passive_monitor_id);
              assert.equal(6, data.response.tcp_multiplexing_number);
              assert.equal(true, Array.isArray(data.response.active_monitor_ids));
              assert.equal(false, data.response.tcp_multiplexing_enabled);
              assert.equal(1, data.response.min_active_members);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerpools - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerpools('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionsections - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionsections('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.is_default);
              assert.equal(1, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionsections - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionsections('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallexcludelistactionremoveMember - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallexcludelistactionremoveMember('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.target_display_name);
              assert.equal(true, data.response.is_valid);
              assert.equal('string', data.response.target_id);
              assert.equal('string', data.response.target_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLoadbalancerpersistenceProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLoadbalancerpersistenceProfiles('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LbSourceIpPersistenceProfile', data.response.resource_type);
              assert.equal(false, data.response.persistence_shared);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerpersistenceProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerpersistenceProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLoadbalancerserverSslProfilesserverSslProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLoadbalancerserverSslProfilesserverSslProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.session_cache_enabled);
              assert.equal(false, data.response.is_secure);
              assert.equal(true, Array.isArray(data.response.ciphers));
              assert.equal(true, Array.isArray(data.response.protocols));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLoadbalancerserverSslProfilesserverSslProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLoadbalancerserverSslProfilesserverSslProfileId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerserverSslProfilesserverSslProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerserverSslProfilesserverSslProfileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.session_cache_enabled);
              assert.equal(true, data.response.is_secure);
              assert.equal(true, Array.isArray(data.response.ciphers));
              assert.equal(true, Array.isArray(data.response.protocols));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putFirewallexcludelist - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putFirewallexcludelist('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.members));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFirewallexcludelist - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFirewallexcludelist((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.members));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionsectionssectionIdactionrevise - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionsectionssectionIdactionrevise('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.is_default);
              assert.equal(4, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallsectionsactioncreateWithRules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallsectionsactioncreateWithRules('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.stateful);
              assert.equal(false, data.response.is_default);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(2, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
              assert.equal(true, Array.isArray(data.response.rules));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionsectionssectionIdrulesruleIdactionrevise - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionsectionssectionIdrulesruleIdactionrevise('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal('object', typeof data.response._owner);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.description);
              assert.equal('string', data.response.rule_tag);
              assert.equal('IPV4_IPV6', data.response.ip_protocol);
              assert.equal('string', data.response.key_policy_identifier);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(true, data.response.logged);
              assert.equal('string', data.response.comments);
              assert.equal(false, data.response.disabled);
              assert.equal(true, Array.isArray(data.response.sources));
              assert.equal(true, Array.isArray(data.response.services));
              assert.equal('string', data.response.key_policy_name);
              assert.equal('string', data.response.action);
              assert.equal(true, Array.isArray(data.response.destinations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putMdProxiesproxyId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putMdProxiesproxyId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.secret);
              assert.equal(true, Array.isArray(data.response.metadata_server_ca_ids));
              assert.equal(true, Array.isArray(data.response.edge_cluster_member_indexes));
              assert.equal('string', data.response.metadata_server_url);
              assert.equal('string', data.response.attached_logical_port_id);
              assert.equal('string', data.response.edge_cluster_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getMdProxiesproxyId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getMdProxiesproxyId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.secret);
              assert.equal(true, Array.isArray(data.response.metadata_server_ca_ids));
              assert.equal(true, Array.isArray(data.response.edge_cluster_member_indexes));
              assert.equal('string', data.response.metadata_server_url);
              assert.equal('string', data.response.attached_logical_port_id);
              assert.equal('string', data.response.edge_cluster_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteMdProxiesproxyId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteMdProxiesproxyId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionkeyPolicies - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionkeyPolicies('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.is_default);
              assert.equal('ENC_AES_GCM_128', data.response.encrypt_algorithm);
              assert.equal('MAC_ALG_AES_GCM_128', data.response.mac_algorithm);
              assert.equal('REGULAR', data.response.type);
              assert.equal('INTEGRITY_ONLY', data.response.encrypt_type);
              assert.equal(2592000, data.response.rekey_frequency);
              assert.equal('string', data.response.notes);
              assert.equal(true, data.response.is_sys_default);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionkeyPolicies - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionkeyPolicies('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putDhcpserversserverIdstaticBindingsbindingId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putDhcpserversserverIdstaticBindingsbindingId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(86400, data.response.lease_time);
              assert.equal('string', data.response.gateway_ip);
              assert.equal('object', typeof data.response.options);
              assert.equal('string', data.response.ip_address);
              assert.equal('string', data.response.host_name);
              assert.equal('string', data.response.mac_address);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteDhcpserversserverIdstaticBindingsbindingId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteDhcpserversserverIdstaticBindingsbindingId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcpserversserverIdstaticBindingsbindingId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcpserversserverIdstaticBindingsbindingId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(86400, data.response.lease_time);
              assert.equal('string', data.response.gateway_ip);
              assert.equal('object', typeof data.response.options);
              assert.equal('string', data.response.ip_address);
              assert.equal('string', data.response.host_name);
              assert.equal('string', data.response.mac_address);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNetworkEncryptionkeyManagerskeyManagerId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNetworkEncryptionkeyManagerskeyManagerId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.status);
              assert.equal('string', data.response.host);
              assert.equal(7, data.response.port);
              assert.equal('string', data.response.certificate);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionkeyManagerskeyManagerId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionkeyManagerskeyManagerId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.status);
              assert.equal('string', data.response.host);
              assert.equal(2, data.response.port);
              assert.equal('string', data.response.certificate);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteNetworkEncryptionkeyManagerskeyManagerId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteNetworkEncryptionkeyManagerskeyManagerId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionkeyManagers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionkeyManagers('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.status);
              assert.equal('string', data.response.host);
              assert.equal(9, data.response.port);
              assert.equal('string', data.response.certificate);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionkeyManagers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionkeyManagers((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerservicesserviceIdvirtualServersstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerservicesserviceIdvirtualServersstatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putDhcpserverProfilesprofileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putDhcpserverProfilesprofileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.edge_cluster_member_indexes));
              assert.equal('string', data.response.edge_cluster_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcpserverProfilesprofileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcpserverProfilesprofileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.edge_cluster_member_indexes));
              assert.equal('string', data.response.edge_cluster_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteDhcpserverProfilesprofileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteDhcpserverProfilesprofileId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionsectionssectionIdactionlistWithRules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionsectionssectionIdactionlistWithRules('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.is_default);
              assert.equal(8, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
              assert.equal(true, Array.isArray(data.response.rules));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancersslciphersAndProtocols - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancersslciphersAndProtocols('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.ciphers));
              assert.equal(true, Array.isArray(data.response.protocols));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallsectionssectionIdactionrevise - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallsectionssectionIdactionrevise('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.stateful);
              assert.equal(false, data.response.is_default);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(5, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallexcludelistactionaddMember - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallexcludelistactionaddMember('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.target_display_name);
              assert.equal(false, data.response.is_valid);
              assert.equal('string', data.response.target_id);
              assert.equal('string', data.response.target_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFirewallsectionssectionIdrulesstats - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFirewallsectionssectionIdrulesstats('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(6, data.response.result_count);
              assert.equal('string', data.response.section_id);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLoadbalancerapplicationProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLoadbalancerapplicationProfiles('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LbFastTcpProfile', data.response.resource_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerapplicationProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerapplicationProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLoadbalancermonitorsmonitorId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLoadbalancermonitorsmonitorId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LbHttpMonitor', data.response.resource_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancermonitorsmonitorId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancermonitorsmonitorId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LbUdpMonitor', data.response.resource_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLoadbalancermonitorsmonitorId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLoadbalancermonitorsmonitorId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionstatusactionupdateStatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionstatusactionupdateStatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('UNKNOWN', data.response.status);
              assert.equal('string', data.response.context);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLoadbalancerclientSslProfilesclientSslProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLoadbalancerclientSslProfilesclientSslProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.session_cache_enabled);
              assert.equal(300, data.response.session_cache_timeout);
              assert.equal(true, data.response.prefer_server_ciphers);
              assert.equal(true, data.response.is_secure);
              assert.equal(true, Array.isArray(data.response.ciphers));
              assert.equal(true, Array.isArray(data.response.protocols));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerclientSslProfilesclientSslProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerclientSslProfilesclientSslProfileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.session_cache_enabled);
              assert.equal(300, data.response.session_cache_timeout);
              assert.equal(true, data.response.prefer_server_ciphers);
              assert.equal(true, data.response.is_secure);
              assert.equal(true, Array.isArray(data.response.ciphers));
              assert.equal(true, Array.isArray(data.response.protocols));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLoadbalancerclientSslProfilesclientSslProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLoadbalancerclientSslProfilesclientSslProfileId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNetworkEncryptionsectionssectionIdrulesruleId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNetworkEncryptionsectionssectionIdrulesruleId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal('object', typeof data.response._owner);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.description);
              assert.equal('string', data.response.rule_tag);
              assert.equal('IPV4_IPV6', data.response.ip_protocol);
              assert.equal('string', data.response.key_policy_identifier);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(false, data.response.logged);
              assert.equal('string', data.response.comments);
              assert.equal(true, data.response.disabled);
              assert.equal(true, Array.isArray(data.response.sources));
              assert.equal(true, Array.isArray(data.response.services));
              assert.equal('string', data.response.key_policy_name);
              assert.equal('string', data.response.action);
              assert.equal(true, Array.isArray(data.response.destinations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteNetworkEncryptionsectionssectionIdrulesruleId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteNetworkEncryptionsectionssectionIdrulesruleId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionsectionssectionIdrulesruleId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionsectionssectionIdrulesruleId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal('object', typeof data.response._owner);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.description);
              assert.equal('string', data.response.rule_tag);
              assert.equal('IPV4_IPV6', data.response.ip_protocol);
              assert.equal('string', data.response.key_policy_identifier);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(false, data.response.logged);
              assert.equal('string', data.response.comments);
              assert.equal(false, data.response.disabled);
              assert.equal(true, Array.isArray(data.response.sources));
              assert.equal(true, Array.isArray(data.response.services));
              assert.equal('string', data.response.key_policy_name);
              assert.equal('string', data.response.action);
              assert.equal(true, Array.isArray(data.response.destinations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerservicesserviceIdpoolsstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerservicesserviceIdpoolsstatistics('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerservicesserviceIdvirtualServersvirtualServerIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerservicesserviceIdvirtualServersvirtualServerIdstatus('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('DETACHED', data.response.status);
              assert.equal(7, data.response.last_update_timestamp);
              assert.equal('string', data.response.virtual_server_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerservicesserviceIdvirtualServersvirtualServerIdstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerservicesserviceIdvirtualServersvirtualServerIdstatistics('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(1, data.response.last_update_timestamp);
              assert.equal('object', typeof data.response.statistics);
              assert.equal('string', data.response.virtual_server_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallsectionssectionIdrulesruleIdactionrevise - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallsectionssectionIdrulesruleIdactionrevise('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal('object', typeof data.response._owner);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.description);
              assert.equal(true, data.response.is_default);
              assert.equal('IN_OUT', data.response.direction);
              assert.equal('string', data.response.rule_tag);
              assert.equal('IPV4_IPV6', data.response.ip_protocol);
              assert.equal('string', data.response.notes);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(false, data.response.logged);
              assert.equal(false, data.response.disabled);
              assert.equal(true, Array.isArray(data.response.sources));
              assert.equal(true, Array.isArray(data.response.services));
              assert.equal('ALLOW', data.response.action);
              assert.equal(true, data.response.sources_excluded);
              assert.equal(false, data.response.destinations_excluded);
              assert.equal(true, Array.isArray(data.response.destinations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionsectionssectionIdstate - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionsectionssectionIdstate('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('partial_success', data.response.state);
              assert.equal(true, Array.isArray(data.response.details));
              assert.equal(3, data.response.failure_code);
              assert.equal('string', data.response.failure_message);
              assert.equal(8, data.response.revision_desired);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postMdProxies - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postMdProxies('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.secret);
              assert.equal(true, Array.isArray(data.response.metadata_server_ca_ids));
              assert.equal(true, Array.isArray(data.response.edge_cluster_member_indexes));
              assert.equal('string', data.response.metadata_server_url);
              assert.equal('string', data.response.attached_logical_port_id);
              assert.equal('string', data.response.edge_cluster_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getMdProxies - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getMdProxies('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallsectionssectionIdrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallsectionssectionIdrules('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal('object', typeof data.response._owner);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.description);
              assert.equal(true, data.response.is_default);
              assert.equal('IN_OUT', data.response.direction);
              assert.equal('string', data.response.rule_tag);
              assert.equal('IPV4_IPV6', data.response.ip_protocol);
              assert.equal('string', data.response.notes);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(true, data.response.logged);
              assert.equal(true, data.response.disabled);
              assert.equal(true, Array.isArray(data.response.sources));
              assert.equal(true, Array.isArray(data.response.services));
              assert.equal('ALLOW', data.response.action);
              assert.equal(true, data.response.sources_excluded);
              assert.equal(true, data.response.destinations_excluded);
              assert.equal(true, Array.isArray(data.response.destinations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFirewallsectionssectionIdrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFirewallsectionssectionIdrules('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerservicesserviceIdvirtualServersstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerservicesserviceIdvirtualServersstatistics('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionkeyPolicieskeyPolicyIdactionrotate - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionkeyPolicieskeyPolicyIdactionrotate('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putFirewallsectionssectionId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putFirewallsectionssectionId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.stateful);
              assert.equal(true, data.response.is_default);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(8, data.response.rule_count);
              assert.equal('LAYER2', data.response.section_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFirewallsectionssectionId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFirewallsectionssectionId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.stateful);
              assert.equal(true, data.response.is_default);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(5, data.response.rule_count);
              assert.equal('LAYER2', data.response.section_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteFirewallsectionssectionId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteFirewallsectionssectionId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putFirewallsectionssectionIdrulesruleId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putFirewallsectionssectionIdrulesruleId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal('object', typeof data.response._owner);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.description);
              assert.equal(true, data.response.is_default);
              assert.equal('IN_OUT', data.response.direction);
              assert.equal('string', data.response.rule_tag);
              assert.equal('IPV4_IPV6', data.response.ip_protocol);
              assert.equal('string', data.response.notes);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(false, data.response.logged);
              assert.equal(true, data.response.disabled);
              assert.equal(true, Array.isArray(data.response.sources));
              assert.equal(true, Array.isArray(data.response.services));
              assert.equal('DROP', data.response.action);
              assert.equal(false, data.response.sources_excluded);
              assert.equal(true, data.response.destinations_excluded);
              assert.equal(true, Array.isArray(data.response.destinations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteFirewallsectionssectionIdrulesruleId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteFirewallsectionssectionIdrulesruleId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFirewallsectionssectionIdrulesruleId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFirewallsectionssectionIdrulesruleId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal('object', typeof data.response._owner);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.description);
              assert.equal(true, data.response.is_default);
              assert.equal('IN_OUT', data.response.direction);
              assert.equal('string', data.response.rule_tag);
              assert.equal('IPV4_IPV6', data.response.ip_protocol);
              assert.equal('string', data.response.notes);
              assert.equal(true, Array.isArray(data.response.applied_tos));
              assert.equal(true, data.response.logged);
              assert.equal(false, data.response.disabled);
              assert.equal(true, Array.isArray(data.response.sources));
              assert.equal(true, Array.isArray(data.response.services));
              assert.equal('ALLOW', data.response.action);
              assert.equal(true, data.response.sources_excluded);
              assert.equal(true, data.response.destinations_excluded);
              assert.equal(true, Array.isArray(data.response.destinations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getMdProxiesproxyIdlogicalSwitchIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getMdProxiesproxyIdlogicalSwitchIdstatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('DOWN', data.response.proxy_status);
              assert.equal('string', data.response.error_message);
              assert.equal(true, Array.isArray(data.response.transport_nodes));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFirewallsectionssectionIdstate - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFirewallsectionssectionIdstate('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('orphaned', data.response.state);
              assert.equal(true, Array.isArray(data.response.details));
              assert.equal(2, data.response.failure_code);
              assert.equal('string', data.response.failure_message);
              assert.equal(8, data.response.revision_desired);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getMdProxiesproxyIdstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getMdProxiesproxyIdstatistics('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(5, data.response.timestamp);
              assert.equal(true, Array.isArray(data.response.statistics));
              assert.equal('string', data.response.metadata_proxy_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerservicesserviceIdstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerservicesserviceIdstatistics('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.pools));
              assert.equal('string', data.response.service_id);
              assert.equal(true, Array.isArray(data.response.virtual_servers));
              assert.equal(2, data.response.last_update_timestamp);
              assert.equal('object', typeof data.response.statistics);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNetworkEncryptionkeyPoliciesresetencryptType - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.putNetworkEncryptionkeyPoliciesresetencryptType('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerservicesserviceIdpoolsstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerservicesserviceIdpoolsstatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFirewallsectionssectionIdrulesruleIdstats - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFirewallsectionssectionIdrulesruleIdstats('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.packet_count);
              assert.equal('string', data.response.byte_count);
              assert.equal('string', data.response.rule_id);
              assert.equal('string', data.response.total_session_count);
              assert.equal('string', data.response.session_count);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFirewallrulesruleIdstate - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFirewallrulesruleIdstate('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('success', data.response.state);
              assert.equal(true, Array.isArray(data.response.details));
              assert.equal(10, data.response.failure_code);
              assert.equal('string', data.response.failure_message);
              assert.equal(3, data.response.revision_desired);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallexcludelistactioncheckIfExists - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallexcludelistactioncheckIfExists('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.target_display_name);
              assert.equal(false, data.response.is_valid);
              assert.equal('string', data.response.target_id);
              assert.equal('string', data.response.target_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionkeyPolicieskeyPolicyIdactionrevoke - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionkeyPolicieskeyPolicyIdactionrevoke('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLoadbalancerclientSslProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLoadbalancerclientSslProfiles('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.session_cache_enabled);
              assert.equal(300, data.response.session_cache_timeout);
              assert.equal(true, data.response.prefer_server_ciphers);
              assert.equal(true, data.response.is_secure);
              assert.equal(true, Array.isArray(data.response.ciphers));
              assert.equal(true, Array.isArray(data.response.protocols));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerclientSslProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerclientSslProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFirewallstatsactionreset - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postFirewallstatsactionreset('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionsectionssectionIdrulesactioncreateMultiple - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionsectionssectionIdrulesactioncreateMultiple('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.rules));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getDhcpserversserverIdleases - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getDhcpserversserverIdleases('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(10, data.response.timestamp);
              assert.equal(true, Array.isArray(data.response.leases));
              assert.equal('string', data.response.dhcp_server_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLoadbalancermonitors - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLoadbalancermonitors('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LbHttpMonitor', data.response.resource_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancermonitors - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancermonitors('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLoadbalancerservices - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLoadbalancerservices('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.access_log_enabled);
              assert.equal('object', typeof data.response.attachment);
              assert.equal('INFO', data.response.error_log_level);
              assert.equal(true, data.response.enabled);
              assert.equal(true, Array.isArray(data.response.virtual_server_ids));
              assert.equal('SMALL', data.response.size);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerservices - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerservices('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNetworkEncryptionconfig - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNetworkEncryptionconfig('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.allow_mirrored);
              assert.equal(60, data.response.rekey_margin_time);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionconfig - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionconfig((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.allow_mirrored);
              assert.equal(60, data.response.rekey_margin_time);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionsectionssectionIdrulesruleIdstate - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionsectionssectionIdrulesruleIdstate('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('failed', data.response.state);
              assert.equal(true, Array.isArray(data.response.details));
              assert.equal(3, data.response.failure_code);
              assert.equal('string', data.response.failure_message);
              assert.equal(5, data.response.revision_desired);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerservicesserviceIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerservicesserviceIdstatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.pools));
              assert.equal(10, data.response.cpu_usage);
              assert.equal(true, Array.isArray(data.response.active_transport_nodes));
              assert.equal(1, data.response.memory_usage);
              assert.equal('string', data.response.service_id);
              assert.equal(6, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.standby_transport_nodes));
              assert.equal('string', data.response.error_message);
              assert.equal(true, Array.isArray(data.response.virtual_servers));
              assert.equal('UNKNOWN', data.response.service_status);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFirewallstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFirewallstatus((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLoadbalancervirtualServersvirtualServerId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLoadbalancervirtualServersvirtualServerId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('TCP', data.response.ip_protocol);
              assert.equal('object', typeof data.response.server_ssl_profile_binding);
              assert.equal(true, data.response.enabled);
              assert.equal(7, data.response.max_concurrent_connections);
              assert.equal('string', data.response.pool_id);
              assert.equal('string', data.response.application_profile_id);
              assert.equal(true, Array.isArray(data.response.rule_ids));
              assert.equal(8, data.response.max_new_connection_rate);
              assert.equal('string', data.response.persistence_profile_id);
              assert.equal('object', typeof data.response.client_ssl_profile_binding);
              assert.equal('string', data.response.default_pool_member_port);
              assert.equal('string', data.response.ip_address);
              assert.equal('string', data.response.port);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLoadbalancervirtualServersvirtualServerId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLoadbalancervirtualServersvirtualServerId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancervirtualServersvirtualServerId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancervirtualServersvirtualServerId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('TCP', data.response.ip_protocol);
              assert.equal('object', typeof data.response.server_ssl_profile_binding);
              assert.equal(true, data.response.enabled);
              assert.equal(8, data.response.max_concurrent_connections);
              assert.equal('string', data.response.pool_id);
              assert.equal('string', data.response.application_profile_id);
              assert.equal(true, Array.isArray(data.response.rule_ids));
              assert.equal(10, data.response.max_new_connection_rate);
              assert.equal('string', data.response.persistence_profile_id);
              assert.equal('object', typeof data.response.client_ssl_profile_binding);
              assert.equal('string', data.response.default_pool_member_port);
              assert.equal('string', data.response.ip_address);
              assert.equal('string', data.response.port);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postNetworkEncryptionsectionssectionIdactionupdateWithRules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postNetworkEncryptionsectionssectionIdactionupdateWithRules('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(false, data.response.is_default);
              assert.equal(9, data.response.rule_count);
              assert.equal('LAYER3', data.response.section_type);
              assert.equal(true, Array.isArray(data.response.rules));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionsectionssectionIdrulesruleIdstats - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionsectionssectionIdrulesruleIdstats('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response.packets_out);
              assert.equal(3, data.response.bytes_in);
              assert.equal('string', data.response.rule_id);
              assert.equal(3, data.response.bytes_out);
              assert.equal(2, data.response.packets_in);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLoadbalancerpersistenceProfilespersistenceProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLoadbalancerpersistenceProfilespersistenceProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LbCookiePersistenceProfile', data.response.resource_type);
              assert.equal(false, data.response.persistence_shared);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerpersistenceProfilespersistenceProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerpersistenceProfilespersistenceProfileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LbCookiePersistenceProfile', data.response.resource_type);
              assert.equal(true, data.response.persistence_shared);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLoadbalancerpersistenceProfilespersistenceProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLoadbalancerpersistenceProfilespersistenceProfileId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNetworkEncryptionkeyPolicieskeypolicyIdrules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNetworkEncryptionkeyPolicieskeypolicyIdrules('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancerservicesserviceIdpoolspoolIdstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancerservicesserviceIdpoolspoolIdstatistics('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(1, data.response.last_update_timestamp);
              assert.equal('object', typeof data.response.statistics);
              assert.equal('string', data.response.pool_id);
              assert.equal(true, Array.isArray(data.response.members));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLoadbalancervirtualServers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLoadbalancervirtualServers('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('TCP', data.response.ip_protocol);
              assert.equal('object', typeof data.response.server_ssl_profile_binding);
              assert.equal(true, data.response.enabled);
              assert.equal(1, data.response.max_concurrent_connections);
              assert.equal('string', data.response.pool_id);
              assert.equal('string', data.response.application_profile_id);
              assert.equal(true, Array.isArray(data.response.rule_ids));
              assert.equal(7, data.response.max_new_connection_rate);
              assert.equal('string', data.response.persistence_profile_id);
              assert.equal('object', typeof data.response.client_ssl_profile_binding);
              assert.equal('string', data.response.default_pool_member_port);
              assert.equal('string', data.response.ip_address);
              assert.equal('string', data.response.port);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLoadbalancervirtualServers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLoadbalancervirtualServers('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getEulacontent - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getEulacontent('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.content);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putLicense - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putLicense('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.features);
              assert.equal('string', data.response.description);
              assert.equal('string', data.response.product_version);
              assert.equal(8, data.response.expiry);
              assert.equal(true, data.response.is_eval);
              assert.equal(false, data.response.is_mh);
              assert.equal('string', data.response.license_key);
              assert.equal(true, data.response.is_expired);
              assert.equal('string', data.response.product_name);
              assert.equal('USER', data.response.capacity_type);
              assert.equal(7, data.response.quantity);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLicense - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLicense((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.features);
              assert.equal('string', data.response.description);
              assert.equal('string', data.response.product_version);
              assert.equal(1, data.response.expiry);
              assert.equal(true, data.response.is_eval);
              assert.equal(false, data.response.is_mh);
              assert.equal('string', data.response.license_key);
              assert.equal(true, data.response.is_expired);
              assert.equal('string', data.response.product_name);
              assert.equal('VM', data.response.capacity_type);
              assert.equal(1, data.response.quantity);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getEulaacceptance - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getEulaacceptance((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, data.response.acceptance);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postUpgradeeulaaccept - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postUpgradeeulaaccept((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLicenseslicensesUsage - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLicenseslicensesUsage((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(true, Array.isArray(data.response.feature_usage_info));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLicenseslicenseKey - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLicenseslicenseKey('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.features);
              assert.equal('string', data.response.description);
              assert.equal('string', data.response.product_version);
              assert.equal(6, data.response.expiry);
              assert.equal(false, data.response.is_eval);
              assert.equal(true, data.response.is_mh);
              assert.equal('string', data.response.license_key);
              assert.equal(true, data.response.is_expired);
              assert.equal('string', data.response.product_name);
              assert.equal('USER', data.response.capacity_type);
              assert.equal(10, data.response.quantity);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteLicenseslicenseKey - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteLicenseslicenseKey('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postEulaaccept - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postEulaaccept((data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLicenseslicensesUsageformatcsv - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLicenseslicensesUsageformatcsv((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.file_name);
              assert.equal(3, data.response.last_update_timestamp);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeeulacontent - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeeulacontent('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.content);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLicensesactiondelete - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postLicensesactiondelete('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postLicenses - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postLicenses('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.features);
              assert.equal('string', data.response.description);
              assert.equal('string', data.response.product_version);
              assert.equal(4, data.response.expiry);
              assert.equal(true, data.response.is_eval);
              assert.equal(false, data.response.is_mh);
              assert.equal('string', data.response.license_key);
              assert.equal(false, data.response.is_expired);
              assert.equal('string', data.response.product_name);
              assert.equal('USER', data.response.capacity_type);
              assert.equal(8, data.response.quantity);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLicenses - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLicenses((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUpgradeeulaacceptance - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUpgradeeulaacceptance((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, data.response.acceptance);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointNameservicesnsservicesnsserviceName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointNameservicesnsservicesnsserviceName('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('RealizedNSService', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal(true, Array.isArray(data.response.intent_reference));
              assert.equal('string', data.response.realization_specific_identifier);
              assert.equal('REALIZED', data.response.state);
              assert.equal(true, Array.isArray(data.response.alarms));
              assert.equal('SERVICE', data.response.service_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointNameipSetsipSetsNsxt - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointNameipSetsipSetsNsxt('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradeploymentZonesdeploymentZoneIdenforcementPoints - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradeploymentZonesdeploymentZoneIdenforcementPoints('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointNamefirewallsfirewallSectionsfirewallSectionId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointNamefirewallsfirewallSectionsfirewallSectionId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('RealizedFirewallSection', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal(true, Array.isArray(data.response.intent_reference));
              assert.equal('string', data.response.realization_specific_identifier);
              assert.equal('UNAVAILABLE', data.response.state);
              assert.equal(true, Array.isArray(data.response.alarms));
              assert.equal(true, Array.isArray(data.response.rules));
              assert.equal(4, data.response.rule_count);
              assert.equal('LAYER2', data.response.section_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointNamegroupsnsgroups - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointNamegroupsnsgroups('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postInfraservicesserviceIdserviceEntriesserviceEntryId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postInfraservicesserviceIdserviceEntriesserviceEntryId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteInfraservicesserviceIdserviceEntriesserviceEntryId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteInfraservicesserviceIdserviceEntriesserviceEntryId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfraservicesserviceIdserviceEntriesserviceEntryId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfraservicesserviceIdserviceEntriesserviceEntryId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('IPProtocolServiceEntry', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointNamegroupssecuritygroups - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointNamegroupssecuritygroups('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTemplates - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTemplates('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postInfradomainsdomainIddomainDeploymentMapsdomainDeploymentMapId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postInfradomainsdomainIddomainDeploymentMapsdomainDeploymentMapId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradomainsdomainIddomainDeploymentMapsdomainDeploymentMapId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradomainsdomainIddomainDeploymentMapsdomainDeploymentMapId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal('string', data.response.enforcement_point_path);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteInfradomainsdomainIddomainDeploymentMapsdomainDeploymentMapId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteInfradomainsdomainIddomainDeploymentMapsdomainDeploymentMapId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradeploymentZones - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradeploymentZones('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradomainsdomainIdcommunicationMapcommunicationEntries - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradomainsdomainIdcommunicationMapcommunicationEntries('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postInfradomainsdomainIdcommunicationMapactionrevise - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postInfradomainsdomainIdcommunicationMapactionrevise('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointNamemacSetsmacSetsNsxtmacSetName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointNamemacSetsmacSetsNsxtmacSetName('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal(true, Array.isArray(data.response.intent_reference));
              assert.equal('string', data.response.realization_specific_identifier);
              assert.equal('REALIZED', data.response.state);
              assert.equal(true, Array.isArray(data.response.alarms));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointNamegroupssecuritygroupssecuritygroupName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointNamegroupssecuritygroupssecuritygroupName('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('RealizedSecurityGroup', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal(true, Array.isArray(data.response.intent_reference));
              assert.equal('string', data.response.realization_specific_identifier);
              assert.equal('ERROR', data.response.state);
              assert.equal(true, Array.isArray(data.response.alarms));
              assert.equal(true, Array.isArray(data.response.evaluations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postInfraservicesserviceId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postInfraservicesserviceId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfraservicesserviceId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfraservicesserviceId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal(true, Array.isArray(data.response.service_entries));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteInfraservicesserviceId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteInfraservicesserviceId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postInfradeploymentZonesdeploymentZoneIdenforcementPointsenforcementpointId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postInfradeploymentZonesdeploymentZoneIdenforcementPointsenforcementpointId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradeploymentZonesdeploymentZoneIdenforcementPointsenforcementpointId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradeploymentZonesdeploymentZoneIdenforcementPointsenforcementpointId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal('object', typeof data.response.connection_info);
              assert.equal('string', data.response.version);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteInfradeploymentZonesdeploymentZoneIdenforcementPointsenforcementpointId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteInfradeploymentZonesdeploymentZoneIdenforcementPointsenforcementpointId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postInfradomainsdomainIdgroupsgroupId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postInfradomainsdomainIdgroupsgroupId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteInfradomainsdomainIdgroupsgroupId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteInfradomainsdomainIdgroupsgroupId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradomainsdomainIdgroupsgroupId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradomainsdomainIdgroupsgroupId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal('FAILURE', data.response.state);
              assert.equal(true, Array.isArray(data.response.expression));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfracommunicationProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfracommunicationProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointNameipSetsipSetsNsxtipSetName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointNameipSetsipSetsNsxtipSetName('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal(true, Array.isArray(data.response.intent_reference));
              assert.equal('string', data.response.realization_specific_identifier);
              assert.equal('ERROR', data.response.state);
              assert.equal(true, Array.isArray(data.response.alarms));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfracommunicationProfilescommunicationProfileIdcommunicationProfileEntries - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfracommunicationProfilescommunicationProfileIdcommunicationProfileEntries('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postInfra - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postInfra('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfra - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfra((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal('NONE', data.response.connectivity_strategy);
              assert.equal(true, Array.isArray(data.response.communication_profiles));
              assert.equal(true, Array.isArray(data.response.domains));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postInfracommunicationProfilescommunicationProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postInfracommunicationProfilescommunicationProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfracommunicationProfilescommunicationProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfracommunicationProfilescommunicationProfileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal(true, Array.isArray(data.response.communication_profile_entries));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteInfracommunicationProfilescommunicationProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteInfracommunicationProfilescommunicationProfileId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postInfracommunicationProfilescommunicationProfileIdcommunicationProfileEntriescommunicationProfileEntryId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postInfracommunicationProfilescommunicationProfileIdcommunicationProfileEntriescommunicationProfileEntryId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteInfracommunicationProfilescommunicationProfileIdcommunicationProfileEntriescommunicationProfileEntryId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteInfracommunicationProfilescommunicationProfileIdcommunicationProfileEntriescommunicationProfileEntryId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfracommunicationProfilescommunicationProfileIdcommunicationProfileEntriescommunicationProfileEntryId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfracommunicationProfilescommunicationProfileIdcommunicationProfileEntriescommunicationProfileEntryId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal(true, Array.isArray(data.response.services));
              assert.equal('DROP', data.response.action);
              assert.equal(8, data.response.sequence_number);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointNameservicesnsservices - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointNameservicesnsservices('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTemplatestemplateId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postTemplatestemplateId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTemplatestemplateId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTemplatestemplateId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.template_body);
              assert.equal(true, Array.isArray(data.response.template_placeholders));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteTemplatestemplateId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteTemplatestemplateId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfraservicesserviceIdserviceEntries - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfraservicesserviceIdserviceEntries('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointNamefirewallsfirewallSections - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointNamefirewallsfirewallSections('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfraservices - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfraservices('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postInfradomainsdomainId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postInfradomainsdomainId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradomainsdomainId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradomainsdomainId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal('object', typeof data.response.communication_map);
              assert.equal(true, Array.isArray(data.response.groups));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteInfradomainsdomainId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteInfradomainsdomainId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPoints - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPoints('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradeploymentZonesdeploymentZoneId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradeploymentZonesdeploymentZoneId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal(true, Array.isArray(data.response.enforcement_points));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointNamegroupsnsgroupsnsgroupName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointNamegroupsnsgroupsnsgroupName('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('RealizedNSGroup', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal(true, Array.isArray(data.response.intent_reference));
              assert.equal('string', data.response.realization_specific_identifier);
              assert.equal('UNREALIZED', data.response.state);
              assert.equal(true, Array.isArray(data.response.alarms));
              assert.equal(true, Array.isArray(data.response.evaluations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointNamemacSetsmacSetsNsxt - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointNamemacSetsmacSetsNsxt('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradomainsdomainIddomainDeploymentMaps - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradomainsdomainIddomainDeploymentMaps('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfrarealizedStateenforcementPointsenforcementPointName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfrarealizedStateenforcementPointsenforcementPointName('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal(true, Array.isArray(data.response.intent_reference));
              assert.equal('string', data.response.realization_specific_identifier);
              assert.equal('UNREALIZED', data.response.state);
              assert.equal(true, Array.isArray(data.response.alarms));
              assert.equal('object', typeof data.response.realized_firewalls);
              assert.equal('object', typeof data.response.realized_services);
              assert.equal('object', typeof data.response.realized_groups);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postInfradomainsdomainIdcommunicationMapcommunicationEntriescommunicationEntryId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postInfradomainsdomainIdcommunicationMapcommunicationEntriescommunicationEntryId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteInfradomainsdomainIdcommunicationMapcommunicationEntriescommunicationEntryId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteInfradomainsdomainIdcommunicationMapcommunicationEntriescommunicationEntryId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradomainsdomainIdcommunicationMapcommunicationEntriescommunicationEntryId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradomainsdomainIdcommunicationMapcommunicationEntriescommunicationEntryId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal('string', data.response.communication_profile_path);
              assert.equal(false, data.response.logged);
              assert.equal(true, Array.isArray(data.response.destination_groups));
              assert.equal(7, data.response.sequence_number);
              assert.equal(true, Array.isArray(data.response.source_groups));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradomains - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradomains('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postInfradomainsdomainIdcommunicationMap - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postInfradomainsdomainIdcommunicationMap('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradomainsdomainIdcommunicationMap - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradomainsdomainIdcommunicationMap('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.path);
              assert.equal('string', data.response.relative_path);
              assert.equal(true, Array.isArray(data.response.communication_entries));
              assert.equal(1, data.response.precedence);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTemplatestemplateIdactiondeploy - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postTemplatestemplateIdactiondeploy('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getInfradomainsdomainIdgroups - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getInfradomainsdomainIdgroups('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAaarolesrole - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAaarolesrole('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.role);
              assert.equal(true, Array.isArray(data.response.features));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAaauserInfo - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAaauserInfo((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.user_name);
              assert.equal(true, Array.isArray(data.response.roles));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAaavidmgroups - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAaavidmgroups('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAaavidmusers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAaavidmusers('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putAaaroleBindingsbindingId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putAaaroleBindingsbindingId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('remote_user', data.response.type);
              assert.equal('string', data.response.name);
              assert.equal(true, Array.isArray(data.response.roles));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteAaaroleBindingsbindingId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteAaaroleBindingsbindingId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAaaroleBindingsbindingId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAaaroleBindingsbindingId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('remote_user', data.response.type);
              assert.equal('string', data.response.name);
              assert.equal(true, Array.isArray(data.response.roles));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postAaaregistrationToken - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postAaaregistrationToken((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.token);
              assert.equal(true, Array.isArray(data.response.roles));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAaaroles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAaaroles((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postAaavidmsearch - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postAaavidmsearch('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteAaaregistrationTokentoken - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteAaaregistrationTokentoken('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAaaregistrationTokentoken - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAaaregistrationTokentoken('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.token);
              assert.equal(true, Array.isArray(data.response.roles));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postAaaroleBindings - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postAaaroleBindings('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('remote_group', data.response.type);
              assert.equal('string', data.response.name);
              assert.equal(true, Array.isArray(data.response.roles));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAaaroleBindings - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAaaroleBindings('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsvniPools - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsvniPools('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsmacPools - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsmacPools('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postPoolsipSubnets - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postPoolsipSubnets('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.cidr);
              assert.equal(true, Array.isArray(data.response.allocation_ranges));
              assert.equal('string', data.response.block_id);
              assert.equal(10, data.response.size);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsipSubnets - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsipSubnets('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsvtepLabelPools - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsvtepLabelPools('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putPoolsipBlocksblockId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putPoolsipBlocksblockId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.cidr);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsipBlocksblockId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsipBlocksblockId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.cidr);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deletePoolsipBlocksblockId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deletePoolsipBlocksblockId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsmacPoolspoolId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsmacPoolspoolId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.ranges));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postPoolsipBlocks - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postPoolsipBlocks('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.cidr);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsipBlocks - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsipBlocks('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putPoolsvniPoolspoolId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putPoolsvniPoolspoolId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.ranges));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsvniPoolspoolId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsvniPoolspoolId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.ranges));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postPoolsipSubnetssubnetId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postPoolsipSubnetssubnetId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.allocation_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deletePoolsipSubnetssubnetId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deletePoolsipSubnetssubnetId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsipSubnetssubnetId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsipSubnetssubnetId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.cidr);
              assert.equal(true, Array.isArray(data.response.allocation_ranges));
              assert.equal('string', data.response.block_id);
              assert.equal(7, data.response.size);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postPoolsipPools - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postPoolsipPools('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.subnets));
              assert.equal('object', typeof data.response.pool_usage);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsipPools - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsipPools('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putPoolsipPoolspoolId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putPoolsipPoolspoolId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.subnets));
              assert.equal('object', typeof data.response.pool_usage);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postPoolsipPoolspoolId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postPoolsipPoolspoolId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.allocation_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsipPoolspoolId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsipPoolspoolId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.subnets));
              assert.equal('object', typeof data.response.pool_usage);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deletePoolsipPoolspoolId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deletePoolsipPoolspoolId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsvtepLabelPoolspoolId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsvtepLabelPoolspoolId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.ranges));
              assert.equal('object', typeof data.response.pool_usage);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getPoolsipPoolspoolIdallocations - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getPoolsipPoolspoolIdallocations('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postCsmawsgatewayAmis - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postCsmawsgatewayAmis('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.ami_id);
              assert.equal('string', data.response.region_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawsgatewayAmis - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawsgatewayAmis('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawskeyPairs - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawskeyPairs('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawsgatewaysvpcIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawsgatewaysvpcIdstatus('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.gateway_cluster_id);
              assert.equal(true, Array.isArray(data.response.gateway_instances_status));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putCsmawsgatewaysvpcId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putCsmawsgatewaysvpcId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.vpc_id);
              assert.equal('object', typeof data.response.configuration);
              assert.equal('string', data.response.account_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawsgatewaysvpcId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawsgatewaysvpcId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.vpc_id);
              assert.equal('object', typeof data.response.configuration);
              assert.equal('string', data.response.account_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawsregionsregionId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawsregionsregionId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.vpc_stats);
              assert.equal(true, data.response.has_managed_vpc);
              assert.equal('object', typeof data.response.instance_stats);
              assert.equal(true, Array.isArray(data.response.availability_zones));
              assert.equal('object', typeof data.response.gateway_stats);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postCsmawsgatewaysactiondeploy - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postCsmawsgatewaysactiondeploy('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.vpc_id);
              assert.equal('object', typeof data.response.configuration);
              assert.equal('string', data.response.account_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmcsmstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmcsmstatus('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.ip_address);
              assert.equal('string', data.response.version);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.id);
              assert.equal(true, data.response.managed_by_vmware);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawsvpcs - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawsvpcs('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postCsmnsxManagerAccounts - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postCsmnsxManagerAccounts('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.username);
              assert.equal('string', data.response.public_ip);
              assert.equal('string', data.response.private_ip);
              assert.equal('string', data.response.thumbprint);
              assert.equal('string', data.response.tenant_id);
              assert.equal('string', data.response.password);
              assert.equal('string', data.response.fqdn);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmnsxManagerAccounts - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmnsxManagerAccounts((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawsregions - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawsregions('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmvirtualMachinesvirtualMachineId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmvirtualMachinesvirtualMachineId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('AzureVirtualMachine', data.response.resource_type);
              assert.equal(10, data.response.gateway_ha_index);
              assert.equal('string', data.response.nsx_ip);
              assert.equal('UNDEPLOYING', data.response.gateway_status);
              assert.equal(true, data.response.is_gateway);
              assert.equal(true, data.response.is_gateway_active);
              assert.equal(true, Array.isArray(data.response.error_messages));
              assert.equal('UP', data.response.agent_status);
              assert.equal('string', data.response.logical_switch_id);
              assert.equal('string', data.response.logical_switch_display_name);
              assert.equal('string', data.response.private_ip);
              assert.equal('THREAT', data.response.threat_state);
              assert.equal('string', data.response.os_details);
              assert.equal(false, data.response.managed_by_nsx);
              assert.equal('OVERRIDDEN', data.response.quarantine_state);
              assert.equal(true, Array.isArray(data.response.cloud_tags));
              assert.equal('string', data.response.public_ip);
              assert.equal('string', data.response.os_type);
              assert.equal('string', data.response.agent_version);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putCsmawsgatewayAmisregionId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putCsmawsgatewayAmisregionId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.ami_id);
              assert.equal('string', data.response.region_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteCsmawsgatewayAmisregionId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteCsmawsgatewayAmisregionId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawsgatewayAmisregionId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawsgatewayAmisregionId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.ami_id);
              assert.equal('string', data.response.region_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putCsmnsxManagerAccountsaccountId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putCsmnsxManagerAccountsaccountId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.username);
              assert.equal('string', data.response.public_ip);
              assert.equal('string', data.response.private_ip);
              assert.equal('string', data.response.thumbprint);
              assert.equal('string', data.response.tenant_id);
              assert.equal('string', data.response.password);
              assert.equal('string', data.response.fqdn);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteCsmnsxManagerAccountsaccountId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteCsmnsxManagerAccountsaccountId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmnsxManagerAccountsaccountId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmnsxManagerAccountsaccountId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.username);
              assert.equal('string', data.response.public_ip);
              assert.equal('string', data.response.private_ip);
              assert.equal('string', data.response.thumbprint);
              assert.equal('string', data.response.tenant_id);
              assert.equal('string', data.response.password);
              assert.equal('string', data.response.fqdn);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postCsmawsaccountsaccountIdactionsyncInventory - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postCsmawsaccountsaccountIdactionsyncInventory('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putCsmawsaccountsaccountId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putCsmawsaccountsaccountId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.tenant_id);
              assert.equal('object', typeof data.response.instance_stats);
              assert.equal('AZURE', data.response.cloud_type);
              assert.equal(true, Array.isArray(data.response.auth_users));
              assert.equal('object', typeof data.response.status);
              assert.equal('string', data.response.access_key);
              assert.equal(5, data.response.regions_count);
              assert.equal(false, data.response.auth_mechanism_iam);
              assert.equal('string', data.response.gateway_role_name);
              assert.equal('object', typeof data.response.vpc_stats);
              assert.equal('string', data.response.has_managed_vpc);
              assert.equal('string', data.response.secret_key);
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.iam_role_arn);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteCsmawsaccountsaccountId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteCsmawsaccountsaccountId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawsaccountsaccountId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawsaccountsaccountId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.tenant_id);
              assert.equal('object', typeof data.response.instance_stats);
              assert.equal('AWS', data.response.cloud_type);
              assert.equal(true, Array.isArray(data.response.auth_users));
              assert.equal('object', typeof data.response.status);
              assert.equal('string', data.response.access_key);
              assert.equal(7, data.response.regions_count);
              assert.equal(true, data.response.auth_mechanism_iam);
              assert.equal('string', data.response.gateway_role_name);
              assert.equal('object', typeof data.response.vpc_stats);
              assert.equal('string', data.response.has_managed_vpc);
              assert.equal('string', data.response.secret_key);
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.iam_role_arn);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postCsmawsaccounts - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postCsmawsaccounts('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.tenant_id);
              assert.equal('object', typeof data.response.instance_stats);
              assert.equal('AWS', data.response.cloud_type);
              assert.equal(true, Array.isArray(data.response.auth_users));
              assert.equal('object', typeof data.response.status);
              assert.equal('string', data.response.access_key);
              assert.equal(8, data.response.regions_count);
              assert.equal(true, data.response.auth_mechanism_iam);
              assert.equal('string', data.response.gateway_role_name);
              assert.equal('object', typeof data.response.vpc_stats);
              assert.equal('string', data.response.has_managed_vpc);
              assert.equal('string', data.response.secret_key);
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.iam_role_arn);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawsaccounts - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawsaccounts('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal('object', typeof data.response.all_accounts_vpc_stats);
              assert.equal('object', typeof data.response.all_accounts_instance_stats);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmvirtualMachines - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmvirtualMachines('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postCsmawsgatewaysactionundeploy - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postCsmawsgatewaysactionundeploy('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawsvpcsvpcId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawsvpcsvpcId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.gateway_info);
              assert.equal('string', data.response.ami_id);
              assert.equal('object', typeof data.response.instance_stats);
              assert.equal('string', data.response.region_id);
              assert.equal('NSX_MANAGED', data.response.op_status);
              assert.equal('string', data.response.cidr);
              assert.equal(true, data.response.is_management_vpc);
              assert.equal(true, Array.isArray(data.response.transport_zones));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawssubnets - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawssubnets('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawsgateways - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawsgateways('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getCsmawsaccountsaccountIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getCsmawsaccountsaccountIdstatus('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('SYNCING_AWS_VPCS', data.response.inventory_sync_step);
              assert.equal('SYNCED', data.response.inventory_sync_status);
              assert.equal('INVALID', data.response.credentials_status);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricnodesnodeIdmodules - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricnodesnodeIdmodules('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.software_modules));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFabricvirtualMachinesactionupdateTags - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postFabricvirtualMachinesactionupdateTags('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabriccomputeManagerscomputeManagerIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabriccomputeManagerscomputeManagerIdstatus('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('string', data.response.version);
              assert.equal('UP', data.response.connection_status);
              assert.equal(true, Array.isArray(data.response.connection_errors));
              assert.equal(2, data.response.last_sync_time);
              assert.equal('string', data.response.connection_status_details);
              assert.equal(true, Array.isArray(data.response.registration_errors));
              assert.equal('REGISTERED', data.response.registration_status);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricnodesstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricnodesstatus('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabriccomputeCollections - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabriccomputeCollections('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFabricnodesnodeIdactionrestartInventorySync - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postFabricnodesnodeIdactionrestartInventorySync('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFabriccomputeCollectionFabricTemplates - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFabriccomputeCollectionFabricTemplates('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.compute_collection_id);
              assert.equal(true, data.response.auto_install_nsx);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabriccomputeCollectionFabricTemplates - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabriccomputeCollectionFabricTemplates('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricvifs - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricvifs('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricnodesnodeIdnetworkinterfacesinterfaceId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricnodesnodeIdnetworkinterfacesinterfaceId('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('realtime', data.response.source);
              assert.equal('DOWN', data.response.admin_status);
              assert.equal('UP', data.response.link_status);
              assert.equal(true, Array.isArray(data.response.interface_alias));
              assert.equal(true, data.response.ens_enabled);
              assert.equal('VIRTUAL', data.response.interface_type);
              assert.equal('string', data.response.interface_id);
              assert.equal('string', data.response.connected_switch);
              assert.equal(true, data.response.ens_capable);
              assert.equal(4, data.response.mtu);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricdiscoveredNodesnodeExtId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricdiscoveredNodesnodeExtId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._last_sync_time);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response.certificate);
              assert.equal('string', data.response.origin_id);
              assert.equal(true, Array.isArray(data.response.ip_addresses));
              assert.equal('string', data.response.os_version);
              assert.equal('string', data.response.node_type);
              assert.equal('string', data.response.parent_compute_collection);
              assert.equal('string', data.response.os_type);
              assert.equal(true, Array.isArray(data.response.origin_properties));
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.cm_local_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricdiscoveredNodes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricdiscoveredNodes('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(10, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabriccomputeCollectionsccExtId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabriccomputeCollectionsccExtId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._last_sync_time);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response.origin_id);
              assert.equal(true, Array.isArray(data.response.origin_properties));
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.owner_id);
              assert.equal('string', data.response.origin_type);
              assert.equal('string', data.response.cm_local_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricvirtualMachines - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricvirtualMachines('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricnodesnodeIdnetworkinterfaces - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricnodesnodeIdnetworkinterfaces('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFabricnodesnodeIdactionupgradeInfra - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFabricnodesnodeIdactionupgradeInfra('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.discovered_ip_addresses));
              assert.equal(true, Array.isArray(data.response.ip_addresses));
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.fqdn);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFabriccomputeManagers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFabriccomputeManagers('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.credential);
              assert.equal(true, Array.isArray(data.response.origin_properties));
              assert.equal('string', data.response.origin_type);
              assert.equal('string', data.response.server);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabriccomputeManagers - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabriccomputeManagers('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricnodesnodeIdstate - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricnodesnodeIdstate('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('in_progress', data.response.state);
              assert.equal(true, Array.isArray(data.response.details));
              assert.equal(8, data.response.failure_code);
              assert.equal('string', data.response.failure_message);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFabricdiscoveredNodesnodeExtIdactionhostprep - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFabricdiscoveredNodesnodeExtIdactionhostprep('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.discovered_ip_addresses));
              assert.equal(true, Array.isArray(data.response.ip_addresses));
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.fqdn);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabriccomputeManagerscomputeManagerIdstate - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabriccomputeManagerscomputeManagerIdstate('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('failed', data.response.state);
              assert.equal(true, Array.isArray(data.response.details));
              assert.equal(1, data.response.failure_code);
              assert.equal('string', data.response.failure_message);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricnodesnodeIdcapabilities - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricnodesnodeIdcapabilities('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.capabilities));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFabricnodes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFabricnodes('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.discovered_ip_addresses));
              assert.equal(true, Array.isArray(data.response.ip_addresses));
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.fqdn);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricnodes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricnodes('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricnodesnodeIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricnodesnodeIdstatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('DOWN', data.response.mpa_connectivity_status);
              assert.equal(true, Array.isArray(data.response.lcp_connectivity_status_details));
              assert.equal('string', data.response.mpa_connectivity_status_details);
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.software_version);
              assert.equal('EXITING', data.response.maintenance_mode);
              assert.equal(false, data.response.inventory_sync_paused);
              assert.equal('object', typeof data.response.system_status);
              assert.equal(5, data.response.inventory_sync_reenable_time);
              assert.equal('UNKNOWN', data.response.lcp_connectivity_status);
              assert.equal(4, data.response.last_heartbeat_timestamp);
              assert.equal(9, data.response.last_sync_time);
              assert.equal('INSTALL_FAILED', data.response.host_node_deployment_status);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putFabriccomputeCollectionFabricTemplatesfabricTemplateId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putFabriccomputeCollectionFabricTemplatesfabricTemplateId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.compute_collection_id);
              assert.equal(true, data.response.auto_install_nsx);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabriccomputeCollectionFabricTemplatesfabricTemplateId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabriccomputeCollectionFabricTemplatesfabricTemplateId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('string', data.response.compute_collection_id);
              assert.equal(false, data.response.auto_install_nsx);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteFabriccomputeCollectionFabricTemplatesfabricTemplateId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteFabriccomputeCollectionFabricTemplatesfabricTemplateId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putFabriccomputeManagerscomputeManagerId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putFabriccomputeManagerscomputeManagerId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.credential);
              assert.equal(true, Array.isArray(data.response.origin_properties));
              assert.equal('string', data.response.origin_type);
              assert.equal('string', data.response.server);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteFabriccomputeManagerscomputeManagerId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteFabriccomputeManagerscomputeManagerId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabriccomputeManagerscomputeManagerId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabriccomputeManagerscomputeManagerId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.credential);
              assert.equal(true, Array.isArray(data.response.origin_properties));
              assert.equal('string', data.response.origin_type);
              assert.equal('string', data.response.server);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putFabricnodesnodeId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putFabricnodesnodeId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.discovered_ip_addresses));
              assert.equal(true, Array.isArray(data.response.ip_addresses));
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.fqdn);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postFabricnodesnodeId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postFabricnodesnodeId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.discovered_ip_addresses));
              assert.equal(true, Array.isArray(data.response.ip_addresses));
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.fqdn);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricnodesnodeId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricnodesnodeId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.discovered_ip_addresses));
              assert.equal(true, Array.isArray(data.response.ip_addresses));
              assert.equal('string', data.response.external_id);
              assert.equal('string', data.response.fqdn);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteFabricnodesnodeId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteFabricnodesnodeId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getFabricnodesnodeIdnetworkinterfacesinterfaceIdstats - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getFabricnodesnodeIdnetworkinterfacesinterfaceIdstats('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response.tx_dropped);
              assert.equal(3, data.response.rx_packets);
              assert.equal(7, data.response.tx_carrier);
              assert.equal(9, data.response.rx_bytes);
              assert.equal(4, data.response.tx_errors);
              assert.equal('string', data.response.interface_id);
              assert.equal(8, data.response.tx_colls);
              assert.equal(9, data.response.rx_frame);
              assert.equal(1, data.response.rx_errors);
              assert.equal(9, data.response.tx_bytes);
              assert.equal(5, data.response.rx_dropped);
              assert.equal(6, data.response.tx_packets);
              assert.equal('realtime', data.response.source);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putTransportNodestransportnodeId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putTransportNodestransportnodeId('fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.host_switches));
              assert.equal('object', typeof data.response.host_switch_spec);
              assert.equal('string', data.response.node_id);
              assert.equal('DISABLED', data.response.maintenance_mode);
              assert.equal(true, Array.isArray(data.response.transport_zone_endpoints));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTransportNodestransportnodeId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postTransportNodestransportnodeId('fakedata', 'fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteTransportNodestransportnodeId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteTransportNodestransportnodeId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportNodestransportnodeId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportNodestransportnodeId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.host_switches));
              assert.equal('object', typeof data.response.host_switch_spec);
              assert.equal('string', data.response.node_id);
              assert.equal('FORCE_ENABLED', data.response.maintenance_mode);
              assert.equal(true, Array.isArray(data.response.transport_zone_endpoints));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postComputeCollectionTransportNodeTemplates - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postComputeCollectionTransportNodeTemplates('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.host_switch_spec);
              assert.equal(true, Array.isArray(data.response.transport_zone_endpoints));
              assert.equal(true, Array.isArray(data.response.compute_collection_ids));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getComputeCollectionTransportNodeTemplates - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getComputeCollectionTransportNodeTemplates('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(7, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postBridgeEndpoints - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postBridgeEndpoints('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.ha_enable);
              assert.equal('string', data.response.bridge_cluster_id);
              assert.equal(1, data.response.vlan);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getBridgeEndpoints - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getBridgeEndpoints('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportZoneszoneIdsummary - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportZoneszoneIdsummary('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(2, data.response.num_logical_ports);
              assert.equal('string', data.response.transport_zone_id);
              assert.equal(8, data.response.num_logical_switches);
              assert.equal(2, data.response.num_transport_nodes);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putEdgeClustersedgeClusterId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putEdgeClustersedgeClusterId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('UNKNOWN', data.response.member_node_type);
              assert.equal(true, Array.isArray(data.response.cluster_profile_bindings));
              assert.equal(true, Array.isArray(data.response.members));
              assert.equal('PHYSICAL_MACHINE', data.response.deployment_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteEdgeClustersedgeClusterId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteEdgeClustersedgeClusterId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getEdgeClustersedgeClusterId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getEdgeClustersedgeClusterId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('PUBLIC_CLOUD_GATEWAY_NODE', data.response.member_node_type);
              assert.equal(true, Array.isArray(data.response.cluster_profile_bindings));
              assert.equal(true, Array.isArray(data.response.members));
              assert.equal('UNKNOWN', data.response.deployment_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putHostSwitchProfileshostSwitchProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putHostSwitchProfileshostSwitchProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LldpHostSwitchProfile', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.required_capabilities));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getHostSwitchProfileshostSwitchProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getHostSwitchProfileshostSwitchProfileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('LldpHostSwitchProfile', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.required_capabilities));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteHostSwitchProfileshostSwitchProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteHostSwitchProfileshostSwitchProfileId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getBridgeEndpointsendpointIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getBridgeEndpointsendpointIdstatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.active_nodes));
              assert.equal(7, data.response.last_update_timestamp);
              assert.equal('string', data.response.endpoint_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putBridgeEndpointsbridgeendpointId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putBridgeEndpointsbridgeendpointId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.ha_enable);
              assert.equal('string', data.response.bridge_cluster_id);
              assert.equal(10, data.response.vlan);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getBridgeEndpointsbridgeendpointId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getBridgeEndpointsbridgeendpointId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(8, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(5, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, data.response.ha_enable);
              assert.equal('string', data.response.bridge_cluster_id);
              assert.equal(4, data.response.vlan);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteBridgeEndpointsbridgeendpointId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteBridgeEndpointsbridgeendpointId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postEdgeClusters - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postEdgeClusters('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('EDGE_NODE', data.response.member_node_type);
              assert.equal(true, Array.isArray(data.response.cluster_profile_bindings));
              assert.equal(true, Array.isArray(data.response.members));
              assert.equal('PHYSICAL_MACHINE', data.response.deployment_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getEdgeClusters - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getEdgeClusters('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTransportNodestransportnodeIdactionresyncHostConfig - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postTransportNodestransportnodeIdactionresyncHostConfig('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postBridgeClusters - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postBridgeClusters('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.cluster_profile_bindings));
              assert.equal(true, Array.isArray(data.response.bridge_nodes));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getBridgeClusters - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getBridgeClusters('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putClusterProfilesclusterProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putClusterProfilesclusterProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('EdgeHighAvailabilityProfile', data.response.resource_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteClusterProfilesclusterProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteClusterProfilesclusterProfileId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusterProfilesclusterProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusterProfilesclusterProfileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('BridgeHighAvailabilityClusterProfile', data.response.resource_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLldptransportNodesnodeIdinterfacesinterfaceName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLldptransportNodesnodeIdinterfacesinterfaceName('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(6, data.response.ifindex);
              assert.equal(true, Array.isArray(data.response.neighbors));
              assert.equal('string', data.response.mac);
              assert.equal('string', data.response.name);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postEdgeClustersedgeClusterIdactionreplaceTransportNode - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postEdgeClustersedgeClusterIdactionreplaceTransportNode('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('UNKNOWN', data.response.member_node_type);
              assert.equal(true, Array.isArray(data.response.cluster_profile_bindings));
              assert.equal(true, Array.isArray(data.response.members));
              assert.equal('PHYSICAL_MACHINE', data.response.deployment_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getBridgeClustersclusterIdstatus - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getBridgeClustersclusterIdstatus('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(2, data.response.last_update_timestamp);
              assert.equal('UNKNOWN', data.response.health);
              assert.equal('string', data.response.cluster_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getComputeCollectionTransportNodeTemplatestemplateIdstate - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getComputeCollectionTransportNodeTemplatestemplateIdstate('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(8, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postHostSwitchProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postHostSwitchProfiles('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(6, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('UplinkHostSwitchProfile', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.required_capabilities));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getHostSwitchProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getHostSwitchProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTransportZones - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postTransportZones('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('nsxDefaultHostSwitch', data.response.host_switch_name);
              assert.equal('STANDARD', data.response.host_switch_mode);
              assert.equal('OVERLAY', data.response.transport_type);
              assert.equal(true, data.response.nested_nsx);
              assert.equal(true, Array.isArray(data.response.transport_zone_profile_ids));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportZones - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportZones('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putTransportZoneszoneId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putTransportZoneszoneId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(1, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('nsxDefaultHostSwitch', data.response.host_switch_name);
              assert.equal('STANDARD', data.response.host_switch_mode);
              assert.equal('OVERLAY', data.response.transport_type);
              assert.equal(false, data.response.nested_nsx);
              assert.equal(true, Array.isArray(data.response.transport_zone_profile_ids));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportZoneszoneId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportZoneszoneId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('nsxDefaultHostSwitch', data.response.host_switch_name);
              assert.equal('STANDARD', data.response.host_switch_mode);
              assert.equal('OVERLAY', data.response.transport_type);
              assert.equal(true, data.response.nested_nsx);
              assert.equal(true, Array.isArray(data.response.transport_zone_profile_ids));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteTransportZoneszoneId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteTransportZoneszoneId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportNodestransportnodeIdstate - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportNodestransportnodeIdstate('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('pending', data.response.state);
              assert.equal(true, Array.isArray(data.response.details));
              assert.equal(8, data.response.failure_code);
              assert.equal('string', data.response.failure_message);
              assert.equal(true, Array.isArray(data.response.host_switch_states));
              assert.equal('ENABLED', data.response.maintenance_mode_state);
              assert.equal('string', data.response.transport_node_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTransportNodes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postTransportNodes('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(7, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.host_switches));
              assert.equal('object', typeof data.response.host_switch_spec);
              assert.equal('string', data.response.node_id);
              assert.equal('FORCE_ENABLED', data.response.maintenance_mode);
              assert.equal(true, Array.isArray(data.response.transport_zone_endpoints));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportNodes - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportNodes('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postClusterProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postClusterProfiles('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(9, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('BridgeHighAvailabilityClusterProfile', data.response.resource_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getClusterProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getClusterProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(1, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getLldptransportNodesnodeIdinterfaces - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getLldptransportNodesnodeIdinterfaces('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putBridgeClustersbridgeclusterId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putBridgeClustersbridgeclusterId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.cluster_profile_bindings));
              assert.equal(true, Array.isArray(data.response.bridge_nodes));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteBridgeClustersbridgeclusterId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteBridgeClustersbridgeclusterId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getBridgeClustersbridgeclusterId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getBridgeClustersbridgeclusterId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.cluster_profile_bindings));
              assert.equal(true, Array.isArray(data.response.bridge_nodes));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putComputeCollectionTransportNodeTemplatestemplateId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putComputeCollectionTransportNodeTemplatestemplateId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.host_switch_spec);
              assert.equal(true, Array.isArray(data.response.transport_zone_endpoints));
              assert.equal(true, Array.isArray(data.response.compute_collection_ids));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getComputeCollectionTransportNodeTemplatestemplateId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getComputeCollectionTransportNodeTemplatestemplateId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(6, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal('object', typeof data.response.host_switch_spec);
              assert.equal(true, Array.isArray(data.response.transport_zone_endpoints));
              assert.equal(true, Array.isArray(data.response.compute_collection_ids));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteComputeCollectionTransportNodeTemplatestemplateId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteComputeCollectionTransportNodeTemplatestemplateId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getBridgeEndpointsendpointIdstatistics - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getBridgeEndpointsendpointIdstatistics('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response.tx_bytes);
              assert.equal('object', typeof data.response.rx_packets);
              assert.equal('object', typeof data.response.tx_packets);
              assert.equal('object', typeof data.response.rx_bytes);
              assert.equal(6, data.response.last_update_timestamp);
              assert.equal('string', data.response.endpoint_id);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putTransportzoneProfilestransportzoneProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putTransportzoneProfilestransportzoneProfileId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(2, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('BfdHealthMonitoringProfile', data.response.resource_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteTransportzoneProfilestransportzoneProfileId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteTransportzoneProfilestransportzoneProfileId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportzoneProfilestransportzoneProfileId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportzoneProfilestransportzoneProfileId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('BfdHealthMonitoringProfile', data.response.resource_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportNodesstate - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportNodesstate('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postTransportzoneProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postTransportzoneProfiles('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(9, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('BfdHealthMonitoringProfile', data.response.resource_type);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportzoneProfiles - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportzoneProfiles('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(5, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putIpfixconfigsconfigId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putIpfixconfigsconfigId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(10, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('IpfixDfwConfig', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.applied_tos));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteIpfixconfigsconfigId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteIpfixconfigsconfigId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getIpfixconfigsconfigId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getIpfixconfigsconfigId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(8, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('IpfixDfwConfig', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.applied_tos));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postIpfixconfigs - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postIpfixconfigs('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(1, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(4, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('IpfixSwitchConfig', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.applied_tos));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getIpfixconfigs - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getIpfixconfigs('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postIpfixcollectorconfigs - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postIpfixcollectorconfigs('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(6, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(4, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.collectors));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getIpfixcollectorconfigs - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getIpfixcollectorconfigs('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(3, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putIpfixcollectorconfigscollectorConfigId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putIpfixcollectorconfigscollectorConfigId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(5, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.collectors));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteIpfixcollectorconfigscollectorConfigId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteIpfixcollectorconfigscollectorConfigId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getIpfixcollectorconfigscollectorConfigId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getIpfixcollectorconfigscollectorConfigId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(3, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(5, data.response._create_time);
              assert.equal(2, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('string', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.collectors));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNormalizations - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNormalizations('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(6, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getRealizationStateBarriercurrent - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getRealizationStateBarriercurrent((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(1, data.response.current_barrier_number);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postRealizationStateBarriercurrentactionincrement - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postRealizationStateBarriercurrentactionincrement((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(6, data.response.current_barrier_number);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putRealizationStateBarrierconfig - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putRealizationStateBarrierconfig('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(7, data.response._revision);
              assert.equal(5, data.response.interval);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getRealizationStateBarrierconfig - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getRealizationStateBarrierconfig((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(9, data.response._revision);
              assert.equal(6, data.response.interval);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportNodesnodeIdtunnels - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportNodesnodeIdtunnels('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(9, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.tunnels));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTransportNodesnodeIdtunnelstunnelName - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTransportNodesnodeIdtunnelstunnelName('fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('DOWN', data.response.status);
              assert.equal('string', data.response.egress_interface);
              assert.equal('string', data.response.remote_node_display_name);
              assert.equal('string', data.response.remote_node_id);
              assert.equal('VXLAN', data.response.encap);
              assert.equal('object', typeof data.response.bfd);
              assert.equal('string', data.response.name);
              assert.equal('string', data.response.local_ip);
              assert.equal(8, data.response.last_updated_time);
              assert.equal('string', data.response.remote_ip);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getAssociations - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getAssociations('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(true, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(2, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putUiViewsdashboardwidgetconfigurationswidgetconfigurationId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putUiViewsdashboardwidgetconfigurationswidgetconfigurationId('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(10, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(3, data.response._create_time);
              assert.equal(8, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('ContainerConfiguration', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.datasources));
              assert.equal(10000, data.response.weight);
              assert.equal('object', typeof data.response.footer);
              assert.equal(true, data.response.shared);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUiViewsdashboardwidgetconfigurationswidgetconfigurationId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUiViewsdashboardwidgetconfigurationswidgetconfigurationId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(4, data.response._revision);
              assert.equal(false, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(10, data.response._create_time);
              assert.equal(3, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('MultiWidgetConfiguration', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.datasources));
              assert.equal(10000, data.response.weight);
              assert.equal('object', typeof data.response.footer);
              assert.equal(false, data.response.shared);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#deleteUiViewsdashboardwidgetconfigurationswidgetconfigurationId - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.deleteUiViewsdashboardwidgetconfigurationswidgetconfigurationId('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postUiViewsdashboardwidgetconfigurations - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postUiViewsdashboardwidgetconfigurations('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(2, data.response._revision);
              assert.equal(true, data.response._system_owned);
              assert.equal('string', data.response.display_name);
              assert.equal('string', data.response.description);
              assert.equal(true, Array.isArray(data.response.tags));
              assert.equal('string', data.response._create_user);
              assert.equal('string', data.response._protection);
              assert.equal(1, data.response._create_time);
              assert.equal(7, data.response._last_modified_time);
              assert.equal('string', data.response._last_modified_user);
              assert.equal('string', data.response.id);
              assert.equal('DonutConfiguration', data.response.resource_type);
              assert.equal(true, Array.isArray(data.response.datasources));
              assert.equal(10000, data.response.weight);
              assert.equal('object', typeof data.response.footer);
              assert.equal(true, data.response.shared);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getUiViewsdashboardwidgetconfigurations - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getUiViewsdashboardwidgetconfigurations('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.widgetconfigurations));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getErrorResolvererrorId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getErrorResolvererrorId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(6, data.response.error_id);
              assert.equal(true, data.response.resolver_present);
              assert.equal('object', typeof data.response.user_metadata);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getErrorResolver - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getErrorResolver((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postErrorResolveractionresolveError - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.postErrorResolveractionresolveError('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#putNodeaaaauthPolicy - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.putNodeaaaauthPolicy('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(900, data.response.api_failed_auth_reset_period);
              assert.equal(8, data.response.minimum_password_length);
              assert.equal(900, data.response.cli_failed_auth_lockout_period);
              assert.equal(5, data.response.api_max_auth_failures);
              assert.equal(900, data.response.api_failed_auth_lockout_period);
              assert.equal(5, data.response.cli_max_auth_failures);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getNodeaaaauthPolicy - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getNodeaaaauthPolicy((data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal(900, data.response.api_failed_auth_reset_period);
              assert.equal(8, data.response.minimum_password_length);
              assert.equal(900, data.response.cli_failed_auth_lockout_period);
              assert.equal(5, data.response.api_max_auth_failures);
              assert.equal(900, data.response.api_failed_auth_lockout_period);
              assert.equal(5, data.response.cli_max_auth_failures);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTaskstaskId - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTaskstaskId('fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('success', data.response.status);
              assert.equal(false, data.response.async_response_available);
              assert.equal('string', data.response.description);
              assert.equal(2, data.response.start_time);
              assert.equal(false, data.response.cancelable);
              assert.equal('string', data.response.request_method);
              assert.equal('string', data.response.user);
              assert.equal(8, data.response.progress);
              assert.equal('string', data.response.message);
              assert.equal('string', data.response.request_uri);
              assert.equal('string', data.response.id);
              assert.equal(1, data.response.end_time);
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#postBatch - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.postBatch('fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal(false, data.response.has_errors);
              assert.equal(true, data.response.rolled_back);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTasks - errors', () => {
      it('should work if integrated or standalone with mockdata', (done) => {
        const p = new Promise((resolve) => {
          a.getTasks('fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', 'fakedata', (data, error) => {
            resolve(data);
            runCommonAsserts(data, error);

            if (stub) {
              assert.equal('object', typeof data.response._self);
              assert.equal(true, Array.isArray(data.response._links));
              assert.equal('string', data.response._schema);
              assert.equal('string', data.response.cursor);
              assert.equal(false, data.response.sort_ascending);
              assert.equal('string', data.response.sort_by);
              assert.equal(4, data.response.result_count);
              assert.equal(true, Array.isArray(data.response.results));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });

    describe('#getTaskstaskIdresponse - errors', () => {
      it('should work if integrated but since no mockdata should error when run standalone', (done) => {
        const p = new Promise((resolve) => {
          a.getTaskstaskIdresponse('fakedata', (data, error) => {
            resolve(data);

            if (stub) {
              assert.notEqual(undefined, error);
              assert.notEqual(null, error);
              assert.equal(null, data);
              assert.equal('AD.500', error.icode);
              assert.equal('Error 400 received on request', error.IAPerror.displayString);
              const temp = 'no mock data for';
              assert.equal(0, error.IAPerror.raw_response.message.indexOf(temp));
            } else {
              assert.equal(undefined, error);
              assert.notEqual(undefined, data);
              assert.notEqual(null, data);
              assert.notEqual(null, data.response);
            }

            done();
          });
        });
        // log just done to get rid of const lint issue!
        log.debug(p);
      }).timeout(attemptTimeout);
    });
  });
});
